@@ -1,7 +1,6 @@
 package uk.ac.starlink.ttools.jel;
 
 import java.io.IOException;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Random;
 import java.util.logging.Logger;
@@ -139,8 +138,10 @@ public abstract class StarTableJELRowReader extends JELRowReader {
         }
     }
 
-    protected Class getColumnClass( int icol ) {
-        return table_.getColumnInfo( icol ).getContentClass();
+    protected Class<?> getColumnClass( int icol ) {
+        return icol < table_.getColumnCount()
+             ? table_.getColumnInfo( icol ).getContentClass()
+             : null;
     }
 
     protected int getColumnIndexByName( String name ) {
@@ -195,14 +196,13 @@ public abstract class StarTableJELRowReader extends JELRowReader {
      * by Utype (using the {@link #UTYPE_PREFIX} prefix).
      */
     protected Constant getConstantByName( String name ) {
-        List paramList = table_.getParameters();
+        List<DescribedValue> paramList = table_.getParameters();
 
         /* Try it as a UCD specification. */
         String ucdSpec = stripPrefix( name, UCD_PREFIX );
         if ( ucdSpec != null ) {
             Pattern ucdRegex = getUcdRegex( ucdSpec );
-            for ( Iterator it = paramList.iterator(); it.hasNext(); ) {
-                DescribedValue dval = (DescribedValue) it.next();
+            for ( DescribedValue dval : paramList ) {
                 String ucd = dval.getInfo().getUCD();
                 if ( ucd != null && ucdRegex.matcher( ucd ).matches() ) {
                     return createDescribedValueConstant( dval );
@@ -215,8 +215,7 @@ public abstract class StarTableJELRowReader extends JELRowReader {
         String utypeSpec = stripPrefix( name, UTYPE_PREFIX );
         if ( utypeSpec != null ) {
             Pattern utypeRegex = getUtypeRegex( utypeSpec );
-            for ( Iterator it = paramList.iterator(); it.hasNext(); ) {
-                DescribedValue dval = (DescribedValue) it.next();
+            for ( DescribedValue dval : paramList ) {
                 String utype = dval.getInfo().getUtype();
                 if ( utype != null && utypeRegex.matcher( utype ).matches() ) {
                     return createDescribedValueConstant( dval );
@@ -229,14 +228,12 @@ public abstract class StarTableJELRowReader extends JELRowReader {
          * case-insensitive. */
         String pname = stripPrefix( name, PARAM_PREFIX );
         if ( pname != null ) {
-            for ( Iterator it = paramList.iterator(); it.hasNext(); ) {
-                DescribedValue dval = (DescribedValue) it.next();
+            for ( DescribedValue dval : paramList ) {
                 if ( pname.equals( dval.getInfo().getName() ) ) {
                     return createDescribedValueConstant( dval );
                 }
             }
-            for ( Iterator it = paramList.iterator(); it.hasNext(); ) {
-                DescribedValue dval = (DescribedValue) it.next();
+            for ( DescribedValue dval : paramList ) {
                 if ( pname.equalsIgnoreCase( dval.getInfo().getName() ) ) {
                     return createDescribedValueConstant( dval );
                 }
@@ -247,7 +244,7 @@ public abstract class StarTableJELRowReader extends JELRowReader {
         /* Try special values for row and column count. */
         if ( name.equalsIgnoreCase( "$nrow" ) ) {
             return new Constant() {
-                public Class getContentClass() {
+                public Class<?> getContentClass() {
                     return Long.class;
                 }
                 public Object getValue() {
@@ -257,7 +254,7 @@ public abstract class StarTableJELRowReader extends JELRowReader {
         }
         if ( name.equalsIgnoreCase( "$ncol" ) ) {
             return new Constant() {
-                public Class getContentClass() {
+                public Class<?> getContentClass() {
                     return Integer.class;
                 }
                 public Object getValue() {
@@ -286,7 +283,7 @@ public abstract class StarTableJELRowReader extends JELRowReader {
              name.equalsIgnoreCase( "Index" ) ||
              name.equalsIgnoreCase( "$index" ) ) {
             return new Constant() {
-                public Class getContentClass() {
+                public Class<?> getContentClass() {
                     return Long.class;
                 }
                 public Object getValue() {
@@ -297,7 +294,7 @@ public abstract class StarTableJELRowReader extends JELRowReader {
         else if ( name.equalsIgnoreCase( "$random" ) ||
                   name.equals( "RANDOM" ) ) {
             return new Constant() {
-                public Class getContentClass() {
+                public Class<?> getContentClass() {
                     return Double.class;
                 }
                 public Object getValue() {
@@ -321,11 +318,11 @@ public abstract class StarTableJELRowReader extends JELRowReader {
      * @return   constant which evaluates to dval's value
      */
     protected Constant createDescribedValueConstant( DescribedValue dval ) {
-        final Class clazz = dval.getInfo().getContentClass();
+        final Class<?> clazz = dval.getInfo().getContentClass();
         Object val = dval.getValue();
         final Object value = Tables.isBlank( val ) ? null : val;
         return new Constant() {
-            public Class getContentClass() {
+            public Class<?> getContentClass() {
                 return clazz;
             }
             public Object getValue() {
