@@ -72,6 +72,9 @@ public class PlotUtil {
         }
     };
 
+    /** Span instance not initialised with any data. */
+    public static final Span EMPTY_SPAN = new BasicRanger( true ).createSpan();
+
     /** Relative location of latex font location list. */
     private static final String LATEX_FONT_PATHS = "latex_fonts.txt";
 
@@ -119,6 +122,19 @@ public class PlotUtil {
         return o1 == null ? o2 == null : o1.equals( o2 );
     }
 
+    /**
+     * Indicates whether two double values are equivalent.
+     * Unlike the == operator, this function returns true if both are NaN.
+     *
+     * @param  d1  first value
+     * @param  d2  second value
+     * @return   true iff inputs are both equal or both NaN
+     */
+    public static boolean doubleEquals( double d1, double d2 ) {
+        return Double.isNaN( d1 ) ? Double.isNaN( d2 )
+                                  : d1 == d2;
+    }
+
     /**
      * Returns a hash code for a possibly null object.
      *
@@ -236,7 +252,7 @@ public class PlotUtil {
         List<T> list = new ArrayList<T>( count );
         list.addAll( Arrays.asList( a1 ) );
         list.addAll( Arrays.asList( a2 ) );
-        Class eClazz = a1.getClass().getComponentType();
+        Class<?> eClazz = a1.getClass().getComponentType();
         @SuppressWarnings("unchecked")
         T[] result =
             (T[]) list.toArray( (Object[]) Array.newInstance( eClazz, count ) );
@@ -378,6 +394,7 @@ public class PlotUtil {
      * @return   array containing element
      */
     public static <T> T[] singletonArray( T object ) {
+        @SuppressWarnings("unchecked")
         T[] array = (T[]) Array.newInstance( object.getClass(), 1 );
         array[ 0 ] = object;
         return array;
@@ -394,6 +411,7 @@ public class PlotUtil {
     public static <P> P[] createProfileArray( SurfaceFactory<P,?> surfFact,
                                               int length ) {
         P profile = surfFact.createProfile( new ConfigMap() );
+        @SuppressWarnings("unchecked")
         P[] array = (P[]) Array.newInstance( profile.getClass(), length );
         return array;
     }
@@ -411,6 +429,7 @@ public class PlotUtil {
         ConfigMap config = new ConfigMap();
         P profile = surfFact.createProfile( config );
         A aspect = surfFact.createAspect( profile, config, null );
+        @SuppressWarnings("unchecked")
         A[] array = (A[]) Array.newInstance( aspect.getClass(), length );
         return array;
     }
@@ -436,34 +455,26 @@ public class PlotUtil {
     }
 
     /**
-     * Determines range information for a set of layers which have
-     * Cartesian (or similar) coordinates.
-     *
-     * <p>The <code>logFlags</code> array can flag whether
-     * some of the data dimensions have logarithmic scaling.
-     * This may not make sense in all cases, if not, supply null.
+     * Extends existing range objects using range information
+     * for a set of layers which have Cartesian (or similar) coordinates.
      *
      * @param   layers   plot layers
-     * @param   nDataDim  dimensionality of data points
+     * @param   ranges   <code>nDataDim</code>-element array of range objects
+     *                   to extend
      * @param   logFlags  <code>nDataDim</code>-element array indicating
      *                    whether data dimensions are
-     *                    linear (false) or logarithmic (true)
+     *                    linear (false) or logarithmic (true),
+     * @param   doPad    whether to add a small standard amount of padding
+     *                   to the result
      * @param   dataStore  data storage
-     * @return   <code>nDataDim</code>-element array of ranges, each containing
-     *           the range of data position coordinate values for
-     *           the corresponding dimension
      */
     @Slow
-    public static Range[] readCoordinateRanges( PlotLayer[] layers,
-                                                int nDataDim,
-                                                boolean[] logFlags,
-                                                DataStore dataStore ) {
-
-        /* Set up an array of range objects, one for each data dimension. */
-        Range[] ranges = new Range[ nDataDim ];
-        for ( int idim = 0; idim < nDataDim; idim++ ) {
-            ranges[ idim ] = new Range();
-        }
+    public static void extendCoordinateRanges( PlotLayer[] layers,
+                                               Range[] ranges,
+                                               boolean[] logFlags,
+                                               boolean doPad,
+                                               DataStore dataStore ) {
+        int nDataDim = ranges.length;
 
         /* Create a point cloud containing all the data coordinates
          * represented by the supplied layers.  If there are several
@@ -487,19 +498,16 @@ public class PlotUtil {
 
         /* If any of the layers wants to supply non-data-position points
          * to mark out additional space, take account of those too. */
-        boolean[] lflags = logFlags == null ? new boolean[ nDataDim ]
-                                            : logFlags;
         for ( int il = 0; il < layers.length; il++ ) {
-            layers[ il ].extendCoordinateRanges( ranges, lflags, dataStore );
+            layers[ il ].extendCoordinateRanges( ranges, logFlags, dataStore );
         }
 
         /* Pad the ranges with a bit of space. */
-        for ( int idim = 0; idim < nDataDim; idim++ ) {
-            padRange( ranges[ idim ], logFlags[ idim ] );
+        if ( doPad ) {
+            for ( int idim = 0; idim < nDataDim; idim++ ) {
+                padRange( ranges[ idim ], logFlags[ idim ] );
+            }
         }
-
-        /* Return the ranges. */
-        return ranges;
     }
 
     /**
@@ -512,7 +520,7 @@ public class PlotUtil {
      * @param  logFlag  true for logarithmic scaling, false for linear
      */
     public static void padRange( Range range, boolean logFlag ) {
-        double[] bounds = range.getBounds();
+        double[] bounds = range.getFiniteBounds( logFlag );
         double lo = bounds[ 0 ];
         double hi = bounds[ 1 ];
         if ( lo < hi ) {
@@ -636,7 +644,7 @@ public class PlotUtil {
      *
      * @param  placer  plot placement
      * @param  layers   layers constituting plot content
-     * @param  auxRanges  requested range information calculated from data
+     * @param  auxSpans   requested range information calculated from data
      * @param  dataStore  data storage object
      * @param  paperType  rendering type
      * @param  cached  whether to cache pixels for future use
@@ -645,7 +653,7 @@ public class PlotUtil {
      */ 
     @Slow
     public static Icon createPlotIcon( PlotPlacement placer, PlotLayer[] layers,
-                                       Map<AuxScale,Range> auxRanges,
+                                       Map<AuxScale,Span> auxSpans,
                                        DataStore dataStore, PaperType paperType,
                                        boolean cached,
                                        Collection<Object> storedPlans ) {
@@ -660,7 +668,7 @@ public class PlotUtil {
         }
         for ( int il = 0; il < nl; il++ ) {
             drawings[ il ] = layers[ il ]
-                            .createDrawing( surface, auxRanges, paperType );
+                            .createDrawing( surface, auxSpans, paperType );
             plans[ il ] = drawings[ il ].calculatePlan( knownPlans.toArray(),
                                                         dataStore );
             knownPlans.add( plans[ il ] );
@@ -856,8 +864,23 @@ public class PlotUtil {
     public static double scaleValue( double min, double max, double frac,
                                      boolean isLog ) {
         return isLog
-             ? min * Math.pow( ( max / min ), frac )
-             : min + ( max - min ) * frac;
+             ? Math.exp( Math.log( min ) * ( 1. - frac )
+                       + Math.log( max ) * frac )
+             : min * ( 1. - frac ) + max * frac;
+    }
+
+    /**
+     * Does linear scaling between two values.
+     * This convenience method just calls
+     * <code>scaleValue(min, max, frac, false)</code>
+     *
+     * @param  min  minimum of range
+     * @param  max  maximum of range
+     * @param  frac  fractional scale point
+     * @return   data value corresponding to fractional scale point
+     */
+    public static double scaleValue( double min, double max, double frac ) {
+        return scaleValue( min, max, frac, false );
     }
 
     /**
@@ -898,6 +921,17 @@ public class PlotUtil {
         };
     }
 
+    /**
+     * Returns a basic span instance with a given lower and upper bound.
+     *
+     * @param  lo  lower bound, may be NaN
+     * @param  hi  upper bound, may be NaN
+     * @return  new span
+     */
+    public static Span createSpan( double lo, double hi ) {
+        return EMPTY_SPAN.limit( lo, hi );
+    }
+
     /**
      * Indicates whether two floating point numbers are approximately equal
      * to each other.
@@ -971,12 +1005,12 @@ public class PlotUtil {
             return formatNumber( value, "0.#E0", nsf );
         }
         else if ( epsilon >= 0.9 ) {
-            return Long.toString( (long) Math.round( value ) );
+            return Long.toString( Math.round( value ) );
         }
         else {
             int ndp = (int) Math.round( Math.max( 0, -Math.log10( epsilon ) ) );
             return ndp == 0
-                 ? Long.toString( (long) Math.round( value ) )
+                 ? Long.toString( Math.round( value ) )
                  : formatNumber( value, "0.0", ndp );
         }
     }
