@@ -57,7 +57,8 @@ public class StyleKeys {
             return null;
         }
         public Specifier<MarkShape> createSpecifier() {
-            return new ComboBoxSpecifier<MarkShape>( MarkStyleSelectors
+            return new ComboBoxSpecifier<MarkShape>( MarkShape.class,
+                                                     MarkStyleSelectors
                                                     .createShapeSelector() );
         }
     }.setOptionUsage()
@@ -77,7 +78,8 @@ public class StyleKeys {
             } )
         , 1 ) {
         public Specifier<Integer> createSpecifier() {
-            return new ComboBoxSpecifier<Integer>( MarkStyleSelectors
+            return new ComboBoxSpecifier<Integer>( Integer.class,
+                                                   MarkStyleSelectors
                                                   .createSizeSelector() );
         }
     };
@@ -96,6 +98,7 @@ public class StyleKeys {
             return null;
         }
         public Specifier<XYShape> createSpecifier() {
+            @SuppressWarnings("rawtypes")
             JComboBox shapeSelector = new RenderingComboBox( XYSHAPES ) {
                 @Override
                 protected Icon getRendererIcon( Object shape ) {
@@ -105,7 +108,8 @@ public class StyleKeys {
                     return null;
                 }
             };
-            return new ComboBoxSpecifier<XYShape>( shapeSelector );
+            return new ComboBoxSpecifier<XYShape>( XYShape.class,
+                                                   shapeSelector );
         }
     }.setOptionUsage()
      .addOptionsXml();
@@ -224,12 +228,15 @@ public class StyleKeys {
                 return null;
             }
             public Specifier<BarStyle.Form> createSpecifier() {
+                @SuppressWarnings("rawtypes")
                 JComboBox formSelector = new RenderingComboBox( BARFORMS ) {
                     protected Icon getRendererIcon( Object form ) {
                         return BarStyles.getIcon( (BarStyle.Form) form );
                     }
                 };
-                return new ComboBoxSpecifier<BarStyle.Form>( formSelector );
+                return
+                    new ComboBoxSpecifier<BarStyle.Form>( BarStyle.Form.class,
+                                                          formSelector );
             }
         }.setOptionUsage()
          .addOptionsXml();
@@ -257,6 +264,7 @@ public class StyleKeys {
                 return fillMode.getDescription();
             }
             public Specifier<FillMode> createSpecifier() {
+                @SuppressWarnings("rawtypes")
                 JComboBox fillSelector = new RenderingComboBox( FILLMODES ) {
                     protected Icon getRendererIcon( Object fillmode ) {
                         return ((FillMode) fillmode)
@@ -264,7 +272,8 @@ public class StyleKeys {
                                            new BasicStroke(), 2 );
                     }
                 };
-                return new ComboBoxSpecifier<FillMode>( fillSelector );
+                return new ComboBoxSpecifier<FillMode>( FillMode.class,
+                                                        fillSelector );
             }
          }.setOptionUsage()
           .addOptionsXml();
@@ -609,8 +618,8 @@ public class StyleKeys {
      * @return  stroke key list
      * @see  #createStroke
      */
-    public static ConfigKey[] getStrokeKeys() {
-        return new ConfigKey[] {
+    public static ConfigKey<?>[] getStrokeKeys() {
+        return new ConfigKey<?>[] {
             THICKNESS,
             DASH,
         };
@@ -630,7 +639,7 @@ public class StyleKeys {
         int thick = config.get( THICKNESS );
         float[] dash = config.get( DASH );
         if ( dash != null && thick != 1 ) {
-            dash = (float[]) dash.clone();
+            dash = dash.clone();
             for ( int i = 0; i < dash.length; i++ ) {
                 dash[ i ] *= thick;
             }
@@ -709,6 +718,7 @@ public class StyleKeys {
         return new IntegerConfigKey( meta, dfltThick ) {
             public Specifier<Integer> createSpecifier() {
                 return new ComboBoxSpecifier<Integer>(
+                               Integer.class,
                                new ThicknessComboBox( 5 ) );
             }
         };
