@@ -300,6 +300,26 @@ public class CubeSurface implements Surface {
         return new double[] { dlos_[ idim ], dhis_[ idim ] };
     }
 
+    /**
+     * Indicates the scaling along the three axes.
+     *
+     * @return  3-element array giving X, Y, Z scaling flags:
+     *          false for linear, true for logarithmic
+     */
+    public boolean[] getLogFlags() {
+        return logFlags_;
+    }
+
+    /**
+     * Indicates which axes are reversed.
+     *
+     * @return  3-element array giving X, Y, Z flip flags;
+     *          true to invert normal plot direction
+     */
+    public boolean[] getFlipFlags() {
+        return flipFlags_;
+    }
+
     /**
      * Maps a data space coordinate to a normalised space coordinate.
      * Normalised coordinates are in the range -1..+1.
@@ -446,6 +466,10 @@ public class CubeSurface implements Surface {
         return null;
     }
 
+    public boolean isContinuousLine( double[] dpos0, double[] dpos1 ) {
+        return true;
+    }
+
     public void paintBackground( Graphics g ) {
         Graphics2D g2 = (Graphics2D) g;
         Color color0 = g2.getColor();
