@@ -10,12 +10,15 @@ import java.awt.Point;
 import java.awt.Rectangle;
 import java.awt.RenderingHints;
 import java.awt.Shape;
+import java.awt.Stroke;
 import java.awt.geom.AffineTransform;
 import java.awt.geom.Area;
 import java.awt.geom.GeneralPath;
 import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
+import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.List;
 import java.util.logging.Logger;
 import skyview.geometry.Projecter;
 import skyview.geometry.Rotater;
@@ -24,6 +27,7 @@ import skyview.geometry.TransformationException;
 import uk.ac.starlink.ttools.func.CoordsRadians;
 import uk.ac.starlink.ttools.plot.Matrices;
 import uk.ac.starlink.ttools.plot2.Captioner;
+import uk.ac.starlink.ttools.plot2.LabelledLine;
 import uk.ac.starlink.ttools.plot2.PlotUtil;
 import uk.ac.starlink.ttools.plot2.Surface;
 
@@ -46,6 +50,7 @@ public class SkySurface implements Surface {
     private final SkyAxisLabeller axLabeller_;
     private final Color gridColor_;
     private final Color axlabelColor_;
+    private final Color scalebarColor_;
     private final boolean sexagesimal_;
     private final double crowd_;
     private final Captioner captioner_;
@@ -62,6 +67,12 @@ public class SkySurface implements Surface {
     private final double gZoom_;
     private final boolean skyFillsBounds_;
     private final boolean isContinuous_;
+    private static final int SEGMENT_NPIX = 3;
+    private static final LabelUnit[] RAD_UNITS = new LabelUnit[] { 
+        new LabelUnit( "\"", Math.PI / ( 180 * 60 * 60 ) ),
+        new LabelUnit( "'", Math.PI / ( 180 * 60 ) ),
+        new LabelUnit( "\u00b0", Math.PI / ( 180 ) ),
+    };      
     private static final Logger logger_ =
         Logger.getLogger( "uk.ac.starlink.ttools.plot2" );
 
@@ -81,6 +92,7 @@ public class SkySurface implements Surface {
      * @param  axLabeller  sky axis labelling object
      * @param  gridColor   colour for grid drawing, or null if no grid
      * @param  axlabelColor  colour for axis labels, or null if no labels
+     * @param  scalebarColor  colour for scale bar, or null if not drawn
      * @param  sexagesimal  whether to use sexagesimal coordinates
      * @param  crowd   tick mark crowding factor, 1 is normal
      * @param  captioner  text rendering object
@@ -89,7 +101,8 @@ public class SkySurface implements Surface {
     public SkySurface( Rectangle plotBounds, Projection projection,
                        double[] rotmat, double zoom, double xoff, double yoff,
                        SkySys viewSystem, SkyAxisLabeller axLabeller,
-                       Color gridColor, Color axlabelColor, boolean sexagesimal,
+                       Color gridColor, Color axlabelColor, Color scalebarColor,
+                       boolean sexagesimal,
                        double crowd, Captioner captioner, boolean antialias ) {
         gxlo_ = plotBounds.x;
         gxhi_ = plotBounds.x + plotBounds.width;
@@ -98,6 +111,7 @@ public class SkySurface implements Surface {
         viewSystem_ = viewSystem;
         gridColor_ = gridColor;
         axlabelColor_ = axlabelColor;
+        scalebarColor_ = scalebarColor;
         sexagesimal_ = sexagesimal;
         crowd_ = crowd;
         captioner_ = captioner;
@@ -224,10 +238,88 @@ public class SkySurface implements Surface {
             axLabeller_.createAxisAnnotation( gl, captioner_ )
                        .drawLabels( g2 );
         }
+        if ( scalebarColor_ != null ) {
+            paintScaleBar( g2, scalebarColor_ );
+        }
         g2.setRenderingHint( RenderingHints.KEY_ANTIALIASING, aa0 );
         g2.setColor( color0 );
     }
 
+    /**
+     * Paints a distance scale in the corner of the plot region.
+     *
+     * @param   g  graphics context
+     * @param   fg   scale bar foreground colour
+     */
+    private void paintScaleBar( Graphics2D g2, Color fg ) {
+
+        /* Define approximate start and end points for scale line. */
+        double fstart = 0.05;
+        double fend = 0.20;
+        int x0 = (int) PlotUtil.scaleValue( gxlo_, gxhi_, fstart );
+        int y0 = (int) PlotUtil.scaleValue( gylo_, gyhi_, 1. - fstart );
+        int x1 = (int) PlotUtil.scaleValue( gxlo_, gxhi_, fend, false );
+        Point gp0 = new Point( x0, y0 );
+        Point gp1 = new Point( x1, y0 );
+
+        /* Work out a round number sky distance of about the same extent
+         * as the approximate scale line. */
+        double s0 = screenDistanceRadians( gp0, gp1 );
+        SkyDistance dist = SkyDistance.getRoundDistance( s0 );
+        if ( dist == null ) {
+            return;
+        }
+
+        /* Iteratively determine the endpoints of a line with exactly
+         * (to pixel accuracy) the sky extent of the round number
+         * sky distance. */
+        double distRad = dist.getRadians();
+        int x2 = (int) PlotUtil.scaleValue( x0, x1, distRad / s0 );
+        Point gp2 = new Point( x2, y0 );
+        boolean isOver = screenDistanceRadians( gp0, gp2 ) > distRad;
+        boolean done = false;
+        for ( ; !done && x2 > x0 + 16 && x2 < gxhi_ - 16;
+              x2 += isOver ? -1 : +1 ) {
+            gp2 = new Point( x2, y0 );
+            done = ( screenDistanceRadians( gp0, gp2 ) > distRad ) ^ isOver;
+        }
+        if ( !done ) {
+            return;
+        }
+
+        /* Paint the scale line with background for contrast. */
+        Color color0 = g2.getColor();
+        Stroke stroke0 = g2.getStroke();
+        g2.setColor( Color.WHITE );
+        g2.setStroke( new BasicStroke( 3, BasicStroke.CAP_SQUARE,
+                                       BasicStroke.JOIN_MITER ) );
+        g2.drawLine( gp0.x, gp0.y, gp2.x, gp2.y );
+        g2.drawLine( gp0.x, gp0.y - 3, gp0.x, gp0.y + 3 );
+        g2.drawLine( gp2.x, gp0.y - 3, gp2.x, gp0.y + 3 );
+        g2.setStroke( new BasicStroke( 1, BasicStroke.CAP_SQUARE,
+                                       BasicStroke.JOIN_MITER ) );
+        g2.setColor( fg );
+        g2.drawLine( gp0.x, gp0.y, gp2.x, gp2.y );
+        g2.drawLine( gp0.x, gp0.y - 3, gp0.x, gp0.y + 3 );
+        g2.drawLine( gp2.x, gp0.y - 3, gp2.x, gp0.y + 3 );
+
+        /* Paint the caption with background for contrast. */
+        String caption = dist.getCaption();
+        g2.setColor( new Color( 0xa0ffffff, true ) );
+        int tx = gp0.x + 5 + captioner_.getPad();
+        int ty = gp0.y - captioner_.getPad();
+        g2.translate( tx, ty );
+        Rectangle bounds = captioner_.getCaptionBounds( caption );
+        g2.fillRect( bounds.x - 5, bounds.y, bounds.width + 10, bounds.height );
+        g2.setColor( fg );
+        captioner_.drawCaption( caption, g2 );
+        g2.translate( -tx, -ty );
+
+        /* Restore graphics context. */
+        g2.setStroke( stroke0 );
+        g2.setColor( color0 );
+    }
+
     public Captioner getCaptioner() {
         return captioner_;
     }
@@ -280,6 +372,15 @@ public class SkySurface implements Surface {
         return yoff_;
     }
 
+    /**
+     * Returns the sky system into which coordinates are projected.
+     *
+     * @return  view sky system
+     */
+    public SkySys getViewSystem() {
+        return viewSystem_;
+    }
+
     /**
      * Attempts to construct a GridLiner object which can
      * draw default grid lines on this plot.
@@ -522,6 +623,24 @@ public class SkySurface implements Surface {
         return null;
     }
 
+    /**
+     * Attempts to turn a graphics position into a sky position.
+     * This convenience function just calls
+     * <code>graphicsToData(gpos,null)</code>,
+     * since the iterable argument is ignored for the SkySurface.
+     *
+     * @param   gpos  graphics position
+     * @return   3-element unit vector representing data position,
+     *           or null if gpos out of range
+     */
+    public double[] graphicsToData( Point2D gpos ) {
+        return graphicsToData( gpos, null );
+    }
+
+    public boolean isContinuousLine( double[] dpos0, double[] dpos1 ) {
+        return projection_.isContinuousLine( dpos0, dpos1 );
+    }
+
     public String formatPosition( double[] dpos ) {
         double pixRad = 2.0 * Math.PI / gZoom_;
         double x = dpos[ 0 ];
@@ -613,14 +732,84 @@ public class SkySurface implements Surface {
      *          or NaN if one of the positions is not on the sky
      */
     public double screenDistanceRadians( Point2D gp1, Point2D gp2 ) {
-        double[] dp1 = graphicsToData( gp1, null );
-        double[] dp2 = graphicsToData( gp2, null );
+        double[] dp1 = graphicsToData( gp1 );
+        double[] dp2 = graphicsToData( gp2 );
         return dp1 == null || dp2 == null
              ? Double.NaN
              : Math.atan2( Matrices.mod( Matrices.cross( dp1, dp2 ) ),
                            Matrices.dot( dp1, dp2 ) );
     }
 
+    /**
+     * Returns a labelled line corresponding to the (shorter) great circle
+     * arc between two graphics points.
+     *
+     * @param   gp1  start point in graphics space
+     * @param   gp2  end point in graphics space
+     * @return  labelled line, or null if it can't be done, for instance
+     *          if not both points are on the sky
+     */
+    public LabelledLine createLine( Point2D gp1, Point2D gp2 ) {
+
+        /* Get sky positions. */
+        double[] dp1 = graphicsToData( gp1 );
+        double[] dp2 = graphicsToData( gp2 );
+        if ( dp1 == null || dp2 == null ) {
+            return null;
+        }
+
+        /* Get label indicating distance in human-readable terms. */
+        double distRad = screenDistanceRadians( gp1, gp2 );
+        double epsRad = Math.sqrt( Math.max( pixelAreaSteradians( gp1 ),
+                                             pixelAreaSteradians( gp2 ) ) );
+        String label = Double.isNaN( distRad ) || Double.isNaN( epsRad )
+                     ? null
+                     : LabelUnit.formatValue( distRad, epsRad, RAD_UNITS );
+
+        /* Work out the number of segments we will need to have a reasonably
+         * smooth line - average one point every few pixels. */
+        double gdist = Math.hypot( gp2.getX() - gp1.getX(),
+                                   gp2.getY() - gp1.getY() );
+        int np = Math.max( 2, (int) Math.ceil( gdist / SEGMENT_NPIX ) );
+
+        /* Assemble a list of the required number of vertices
+         * by taking points spaced equally between the end points
+         * through the interior of the unit sphere (and rescaling
+         * vectors to unit extent as required). */
+        List<Point2D.Double> pList = new ArrayList<Point2D.Double>( np );
+        double[] dpLast = null;
+        double scale = 1.0 / ( np - 1 );
+        for ( int ip = 0; ip < np; ip++ ) {
+            double frac = ip * scale;
+            double dx = PlotUtil.scaleValue( dp1[ 0 ], dp2[ 0 ], frac );
+            double dy = PlotUtil.scaleValue( dp1[ 1 ], dp2[ 1 ], frac );
+            double dz = PlotUtil.scaleValue( dp1[ 2 ], dp2[ 2 ], frac );
+            double r = Math.sqrt( dx * dx + dy * dy + dz * dz );
+            if ( r > 0 ) {
+                double[] dp = new double[] { dx / r, dy / r, dz / r };
+                Point2D.Double gpos = new Point2D.Double();
+                if ( dataToGraphics( dp, false, gpos ) ) {
+                    if ( dp != null && dpLast != null &&
+                         ! projection_.isContinuousLine( dp, dpLast ) ) {
+                        pList.add( null );
+                    }
+                    pList.add( gpos );
+                }
+                else {
+                    return new LabelledLine( gp1, gp2, label );
+                }
+                dpLast = dp;
+            }
+        }
+
+        /* Turn the vertex list into a LabelledLine; if it didn't work for
+         * some reason, fall back to a straight line in graphics space. */
+        Point2D[] points = pList.size() >= 2
+                         ? pList.toArray( new Point2D.Double[ 0 ] )
+                         : new Point2D[] { gp1, gp2 };
+        return new LabelledLine( points, label );
+    }
+
     /**
      * Formats a lon/lat position as sexagesimal given the approximate size
      * of a screen pixel.
@@ -630,8 +819,8 @@ public class SkySurface implements Surface {
      * @param  latRad  latitude in radians
      * @param  pixRad  approximate size of a screen pixel in radians
      */
-    private static String formatPositionSex( double lonRad, double latRad,
-                                             double pixRad ) {
+    public static String formatPositionSex( double lonRad, double latRad,
+                                            double pixRad ) {
         int secondDp = getDecimalPlaces( pixRad / Math.PI * 12 * 60 * 60 );
         int arcsecDp = getDecimalPlaces( pixRad / Math.PI * 180 * 60 * 60 );
         String lonSex =
@@ -660,8 +849,8 @@ public class SkySurface implements Surface {
      * @param  latRad  latitude in radians
      * @param  pixRad  approximate size of a screen pixel in radians
      */
-    private static String formatPositionDec( double lonRad, double latRad,
-                                             double pixRad ) {
+    public static String formatPositionDec( double lonRad, double latRad,
+                                            double pixRad ) {
         double lonDeg = lonRad * 180 / Math.PI;
         double latDeg = latRad * 180 / Math.PI;
         double pixDeg = pixRad * 180 / Math.PI;
@@ -888,6 +1077,7 @@ public class SkySurface implements Surface {
                 && PlotUtil.equals( this.axLabeller_, other.axLabeller_ )
                 && PlotUtil.equals( this.gridColor_, other.gridColor_ )
                 && PlotUtil.equals( this.axlabelColor_, other.axlabelColor_ )
+                && PlotUtil.equals( this.scalebarColor_, other.scalebarColor_ )
                 && this.sexagesimal_ == other.sexagesimal_
                 && this.crowd_ == other.crowd_
                 && PlotUtil.equals( this.captioner_, other.captioner_ )
@@ -914,6 +1104,7 @@ public class SkySurface implements Surface {
         code = 23 * code + PlotUtil.hashCode( axLabeller_ );
         code = 23 * code + PlotUtil.hashCode( gridColor_ );
         code = 23 * code + PlotUtil.hashCode( axlabelColor_ );
+        code = 23 * code + PlotUtil.hashCode( scalebarColor_ );
         code = 23 * code + ( sexagesimal_ ? 5 : 13 );
         code = 23 * code + Float.floatToIntBits( (float) crowd_ );
         code = 23 * code + PlotUtil.hashCode( captioner_ );
