@@ -2,15 +2,17 @@ package uk.ac.starlink.ttools.plot2.geom;
 
 import java.awt.Color;
 import java.awt.Rectangle;
+import java.awt.geom.Point2D;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
 import uk.ac.starlink.ttools.plot.Range;
 import uk.ac.starlink.ttools.plot2.Captioner;
+import uk.ac.starlink.ttools.plot2.LabelledLine;
 import uk.ac.starlink.ttools.plot2.Navigator;
 import uk.ac.starlink.ttools.plot2.PlotLayer;
-import uk.ac.starlink.ttools.plot2.PointCloud;
-import uk.ac.starlink.ttools.plot2.SubCloud;
+import uk.ac.starlink.ttools.plot2.PlotMetric;
+import uk.ac.starlink.ttools.plot2.PlotUtil;
 import uk.ac.starlink.ttools.plot2.Surface;
 import uk.ac.starlink.ttools.plot2.SurfaceFactory;
 import uk.ac.starlink.ttools.plot2.config.BooleanConfigKey;
@@ -22,8 +24,12 @@ import uk.ac.starlink.ttools.plot2.config.DoubleConfigKey;
 import uk.ac.starlink.ttools.plot2.config.OptionConfigKey;
 import uk.ac.starlink.ttools.plot2.config.SkySysConfigKey;
 import uk.ac.starlink.ttools.plot2.config.StyleKeys;
+import uk.ac.starlink.ttools.plot2.data.DataSpec;
 import uk.ac.starlink.ttools.plot2.data.DataStore;
+import uk.ac.starlink.ttools.plot2.data.EmptyTupleSequence;
 import uk.ac.starlink.ttools.plot2.data.SkyCoord;
+import uk.ac.starlink.ttools.plot2.data.TupleSequence;
+import uk.ac.starlink.ttools.plot2.data.WrapperTupleSequence;
 
 /**
  * Surface factory for plotting on the surface of the celestial sphere.
@@ -143,7 +149,22 @@ public class SkySurfaceFactory
     /** Config key to control axis label positioning. */
     public static final ConfigKey<SkyAxisLabeller> AXISLABELLER_KEY =
         createAxisLabellerKey();
- 
+
+    /* Config key to control scale bar presence. */
+    public static final ConfigKey<Boolean> SCALEBAR_KEY =
+         new BooleanConfigKey(
+             new ConfigMeta( "scalebar", "Draw Scale Bar" )
+            .setShortDescription( "Draw sky scale bar?" )
+            .setXmlDescription( new String[] {
+                 "<p>If true, a small bar is drawn near the bottom left",
+                 "of the plot annotated with a distance in",
+                 "degrees, arc minutes or arc seconds,",
+                 "to make it easier to determine the size of features",
+                 "on the plot by eye.",
+                 "</p>",
+             } )
+         , true );
+
     /** Config key to determine whether sexagesimal coordinates are used. */
     public static final ConfigKey<Boolean> SEX_KEY =
         new BooleanConfigKey(
@@ -204,23 +225,27 @@ public class SkySurfaceFactory
             } )
         , 1 );
 
+    private static PlotMetric plotMetric_ = new SkyPlotMetric();
+
     public Surface createSurface( Rectangle plotBounds, Profile p,
                                   SkyAspect aspect ) {
         return new SkySurface( plotBounds, p.getProjection(),
                                aspect.getRotation(), aspect.getZoom(),
                                aspect.getOffsetX(), aspect.getOffsetY(),
                                p.viewSystem_, p.axisLabeller_,
-                               p.grid_ ? p.gridColor_ : null, p.axlabelColor_,
+                               p.grid_ ? p.gridColor_ : null,
+                               p.axlabelColor_, p.scalebarColor_,
                                p.sex_, p.crowd_, p.captioner_, p.antialias_ );
     }
 
-    public ConfigKey[] getProfileKeys() {
-        List<ConfigKey> list = new ArrayList<ConfigKey>();
-        list.addAll( Arrays.asList( new ConfigKey[] {
+    public ConfigKey<?>[] getProfileKeys() {
+        List<ConfigKey<?>> list = new ArrayList<ConfigKey<?>>();
+        list.addAll( Arrays.asList( new ConfigKey<?>[] {
             PROJECTION_KEY,
             VIEWSYS_KEY,
             REFLECT_KEY,
             GRID_KEY,
+            SCALEBAR_KEY,
             AXISLABELLER_KEY,
             SEX_KEY,
             CROWD_KEY,
@@ -229,7 +254,7 @@ public class SkySurfaceFactory
             StyleKeys.GRID_ANTIALIAS,
         } ) );
         list.addAll( Arrays.asList( StyleKeys.CAPTIONER.getKeys() ) );
-        return list.toArray( new ConfigKey[ 0 ] );
+        return list.toArray( new ConfigKey<?>[ 0 ] );
     }
 
     public Profile createProfile( ConfigMap config ) {
@@ -242,15 +267,16 @@ public class SkySurfaceFactory
         double crowd = config.get( CROWD_KEY );
         Color gridColor = config.get( StyleKeys.GRID_COLOR );
         Color axlabelColor = config.get( StyleKeys.AXLABEL_COLOR );
+        Color scalebarColor = config.get( SCALEBAR_KEY ) ? axlabelColor : null;
         boolean antialias = config.get( StyleKeys.GRID_ANTIALIAS );
         Captioner captioner = StyleKeys.CAPTIONER.createValue( config );
         return new Profile( proj, reflect, viewSystem, grid, axLabeller,
-                            gridColor, axlabelColor, sex, crowd, captioner,
-                            antialias );
+                            gridColor, axlabelColor, scalebarColor,
+                            sex, crowd, captioner, antialias );
     }
 
-    public ConfigKey[] getAspectKeys() {
-        return new ConfigKey[] {
+    public ConfigKey<?>[] getAspectKeys() {
+        return new ConfigKey<?>[] {
             LON_KEY, LAT_KEY, FOV_RADIUS_KEY,
         };
     }
@@ -292,26 +318,52 @@ public class SkySurfaceFactory
     }
 
     public Range[] readRanges( Profile profile, PlotLayer[] layers,
-                               DataStore dataStore ) {
-        PointCloud pointCloud =
-            new PointCloud( SubCloud.createSubClouds( layers, true ) );
-        Range[] ranges = new Range[] { new Range(), new Range(), new Range() };
-        long ip = 0;
-        for ( double[] dpos : pointCloud.createDataPosIterable( dataStore ) ) {
-            for ( int idim = 0; idim < 3; idim++ ) {
-                ranges[ idim ].submit( dpos[ idim ] );
-            }
+                               final DataStore dataStore ) {
 
-            /* Periodically check if the whole sky is covered.
-             * If so, don't bother carrying on. */
-            if ( ++ip % 10000 == 0 && isAllSky( ranges ) ) {
-                return ranges;
+        /* We will call a utility method to turn the layer data into ranges,
+         * which in turn are used to auto-range the data so presenting only
+         * part of the sky for viewing (auto-ranged aspect).
+         * But modify the way the data is passed to it: provide a DataStore
+         * implementation that stops dispensing data when enough of the sky
+         * is covered that looking at more of it will not make a significant
+         * difference to the results.  This avoids doing more work than
+         * we have to. */
+        final Range[] ranges = { new Range(), new Range(), new Range() };
+        DataStore lazyStore = new DataStore() {
+            public boolean hasData( DataSpec dataSpec ) {
+                return dataStore.hasData( dataSpec );
             }
-        }
+            public TupleSequence getTupleSequence( DataSpec dataSpec ) {
+                if ( isFullSky() ) {
+                    return new EmptyTupleSequence();
+                }
+                else {
+                    final TupleSequence baseSeq =
+                         dataStore.getTupleSequence( dataSpec );
+                    return new WrapperTupleSequence( baseSeq ) {
+                        int ip = 0;
+                        @Override
+                        public boolean next() {
+                            return ( ip++ % 10000 == 0 && isFullSky() )
+                                 ? false
+                                 : baseSeq.next();
+                        }
+                    };
+                }
+            }
+            private boolean isFullSky() {
+                return ranges[ 0 ].isFinite()
+                    && ranges[ 1 ].isFinite()
+                    && ranges[ 2 ].isFinite()
+                    && isAllSky( ranges );
+            }
+        };
+        PlotUtil.extendCoordinateRanges( layers, ranges, new boolean[ 3 ],
+                                         false, lazyStore );
         return ranges;
     }
 
-    public ConfigKey[] getNavigatorKeys() {
+    public ConfigKey<?>[] getNavigatorKeys() {
         return SkyNavigator.getConfigKeys();
     }
 
@@ -319,6 +371,10 @@ public class SkySurfaceFactory
         return SkyNavigator.createNavigator( navConfig );
     }
 
+    public PlotMetric getPlotMetric() {
+        return plotMetric_;
+    }
+
     /**
      * Determines whether a set of ranges of normalised data coordinates
      * cover enough of the full data cube (-1..+1 in each dimension)
@@ -350,8 +406,9 @@ public class SkySurfaceFactory
         Projection[] projections = new Projection[] {
             // new HemisphereProjection(),  // toy projection, inferior to Sin
             SinProjection.INSTANCE,
-            SkyviewProjection.AIT,
-            SkyviewProjection.CAR1,
+            FixedSkyviewProjection.AIT,
+            FixedSkyviewProjection.CAR1,
+            FixedSkyviewProjection.CAR0,
         };
         meta.setShortDescription( "Sky coordinate projection" );
         meta.setXmlDescription( new String[] {
@@ -362,7 +419,9 @@ public class SkySurfaceFactory
                 new OptionConfigKey<Projection>( meta, Projection.class,
                                                  projections ) {
             public String valueToString( Projection proj ) {
-                return proj.getProjectionName().toLowerCase();
+                return proj == null
+                     ? null
+                     : proj.getProjectionName().toLowerCase();
             }
             public String getXmlDescription( Projection proj ) {
                 return proj.getProjectionDescription();
@@ -421,6 +480,24 @@ public class SkySurfaceFactory
         return key;
     }
 
+    /**
+     * PlotMetric implementation for the sky surface.
+     */
+    private static class SkyPlotMetric implements PlotMetric {
+        public LabelledLine[] getMeasures( Surface surf,
+                                           Point2D gp0, Point2D gp1 ) {
+            if ( surf instanceof SkySurface ) {
+                LabelledLine line = ((SkySurface) surf).createLine( gp0, gp1 );
+                return line == null
+                     ? new LabelledLine[ 0 ]
+                     : new LabelledLine[] { line };
+            }
+            else {
+                return new LabelledLine[ 0 ];
+            }
+        }
+    }
+
     /**
      * Profile class which defines fixed configuration items for a SkySurface.
      * Instances of this class are normally obtained from the
@@ -434,6 +511,7 @@ public class SkySurfaceFactory
         private final SkyAxisLabeller axisLabeller_;
         private final Color gridColor_;
         private final Color axlabelColor_;
+        private final Color scalebarColor_;
         private final boolean sex_;
         private final double crowd_;
         private final Captioner captioner_;
@@ -449,6 +527,7 @@ public class SkySurfaceFactory
          * @param  axisLabeller  sky axis labelling object
          * @param  gridColor   colour of grid lines
          * @param  axlabelColor  colour of axis labels
+         * @param  scalebarColor  colour of scale bar
          * @param  sex  whether to use sexagesimal coordinates
          * @param  crowd   tick mark crowding factor, 1 is normal
          * @param  captioner  text rendering object
@@ -457,7 +536,7 @@ public class SkySurfaceFactory
         public Profile( Projection projection, boolean reflect,
                         SkySys viewSystem, boolean grid, 
                         SkyAxisLabeller axisLabeller, Color gridColor,
-                        Color axlabelColor, boolean sex,
+                        Color axlabelColor, Color scalebarColor, boolean sex,
                         double crowd, Captioner captioner, boolean antialias ) {
             projection_ = projection;
             reflect_ = reflect;
@@ -466,6 +545,7 @@ public class SkySurfaceFactory
             axisLabeller_ = axisLabeller;
             gridColor_ = gridColor;
             axlabelColor_ = axlabelColor;
+            scalebarColor_ = scalebarColor;
             sex_ = sex;
             crowd_ = crowd;
             captioner_ = captioner;
