@@ -2,10 +2,11 @@ package uk.ac.starlink.ttools.plot2.layer;
 
 import java.awt.geom.Point2D;
 import uk.ac.starlink.table.ValueInfo;
-import uk.ac.starlink.ttools.plot.Range;
 import uk.ac.starlink.ttools.plot2.AuxReader;
 import uk.ac.starlink.ttools.plot2.DataGeom;
 import uk.ac.starlink.ttools.plot2.PlotUtil;
+import uk.ac.starlink.ttools.plot2.Ranger;
+import uk.ac.starlink.ttools.plot2.Scaling;
 import uk.ac.starlink.ttools.plot2.Surface;
 import uk.ac.starlink.ttools.plot2.data.DataSpec;
 import uk.ac.starlink.ttools.plot2.data.DataStore;
@@ -25,8 +26,7 @@ public class FloatingCoordAuxReader implements AuxReader {
     private final int icol_;
     private final DataGeom geom_;
     private final boolean visibleOnly_;
-    private final double[] dpos_;
-    private final Point2D.Double gpos_;
+    private final Scaling scaling_;
 
     /**
      * Constructor.
@@ -37,15 +37,16 @@ public class FloatingCoordAuxReader implements AuxReader {
      * @param  visibleOnly  true to include only points visible in the
      *                      current plot bounds, false for all potentially
      *                      plottable points
+     * @param  scaling  scaling
      */
     public FloatingCoordAuxReader( FloatingCoord coord, int icol,
-                                   DataGeom geom, boolean visibleOnly ) {
+                                   DataGeom geom, boolean visibleOnly,
+                                   Scaling scaling ) {
         coord_ = coord;
         icol_ = icol;
         geom_ = geom;
         visibleOnly_ = visibleOnly;
-        dpos_ = new double[ geom.getDataDimCount() ];
-        gpos_ = new Point2D.Double();
+        scaling_ = scaling;
     }
 
     public int getCoordIndex() {
@@ -57,23 +58,33 @@ public class FloatingCoordAuxReader implements AuxReader {
         return infos.length == 1 ? infos[ 0 ] : null;
     }
 
+    public Scaling getScaling() {
+        return scaling_;
+    }
+
     public void adjustAuxRange( Surface surface, DataSpec dataSpec,
                                 DataStore dataStore, Object[] plans,
-                                Range range ) {
-
-        /* Convert data to graphics coordinates.  The resulting values are
-         * not used, but this determines whether the points are plottable. */
+                                Ranger ranger ) {
         TupleSequence tseq = dataStore.getTupleSequence( dataSpec );
-        while ( tseq.next() ) {
-            if ( geom_.readDataPos( tseq, 0, dpos_ ) &&
-                 surface.dataToGraphics( dpos_, visibleOnly_, gpos_ ) &&
-                 ( visibleOnly_ || PlotUtil.isPointFinite( gpos_ ) ) ) {
 
-                /* Read the coordinate value. */
-                double value = coord_.readDoubleCoord( tseq, icol_ );
+        /* If no positional coordinates, just submit every value. */
+        if ( geom_ == null ) {
+            while ( tseq.next() ) {
+                ranger.submitDatum( coord_.readDoubleCoord( tseq, icol_ ) );
+            }
+        }
 
-                /* Extend the submitted range accordingly. */
-                range.submit( value );
+        /* If there are positional coordinates, check each value to see
+         * whether it is plottable, and only submit the ones that are. */
+        else {
+            double[] dpos = new double[ geom_.getDataDimCount() ];
+            Point2D.Double gpos = new Point2D.Double();
+            while ( tseq.next() ) {
+                if ( geom_.readDataPos( tseq, 0, dpos ) &&
+                     surface.dataToGraphics( dpos, visibleOnly_, gpos ) &&
+                     ( visibleOnly_ || PlotUtil.isPointFinite( gpos ) ) ) {
+                    ranger.submitDatum( coord_.readDoubleCoord( tseq, icol_ ) );
+                }
             }
         }
     }
