@@ -16,7 +16,6 @@ import javax.swing.Icon;
 import uk.ac.starlink.table.DefaultValueInfo;
 import uk.ac.starlink.table.ValueInfo;
 import uk.ac.starlink.ttools.gui.ResourceIcon;
-import uk.ac.starlink.ttools.plot.Range;
 import uk.ac.starlink.ttools.plot.Shader;
 import uk.ac.starlink.ttools.plot.Shaders;
 import uk.ac.starlink.ttools.plot.Style;
@@ -30,11 +29,13 @@ import uk.ac.starlink.ttools.plot2.LayerOpt;
 import uk.ac.starlink.ttools.plot2.PlotLayer;
 import uk.ac.starlink.ttools.plot2.PlotUtil;
 import uk.ac.starlink.ttools.plot2.Plotter;
+import uk.ac.starlink.ttools.plot2.Ranger;
 import uk.ac.starlink.ttools.plot2.ReportKey;
 import uk.ac.starlink.ttools.plot2.ReportMap;
 import uk.ac.starlink.ttools.plot2.ReportMeta;
 import uk.ac.starlink.ttools.plot2.Scaler;
 import uk.ac.starlink.ttools.plot2.Scaling;
+import uk.ac.starlink.ttools.plot2.Span;
 import uk.ac.starlink.ttools.plot2.Subrange;
 import uk.ac.starlink.ttools.plot2.Surface;
 import uk.ac.starlink.ttools.plot2.config.ConfigKey;
@@ -187,8 +188,8 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
         return sbuf.toString();
     }
 
-    public ConfigKey[] getStyleKeys() {
-        List<ConfigKey> keyList = new ArrayList<ConfigKey>();
+    public ConfigKey<?>[] getStyleKeys() {
+        List<ConfigKey<?>> keyList = new ArrayList<ConfigKey<?>>();
         keyList.add( XBINSIZER_KEY );
         keyList.add( YBINSIZER_KEY );
         keyList.add( COMBINER_KEY );
@@ -200,7 +201,7 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
         }
         keyList.add( XPHASE_KEY );
         keyList.add( YPHASE_KEY );
-        return keyList.toArray( new ConfigKey[ 0 ] );
+        return keyList.toArray( new ConfigKey<?>[ 0 ] );
     }
 
     public GridStyle createStyle( ConfigMap config ) {
@@ -211,10 +212,11 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
         Combiner combiner = config.get( COMBINER_KEY );
         RampKeySet.Ramp ramp = RAMP_KEYS.createValue( config );
         Scaling scaling = ramp.getScaling();
+        Subrange dataclip = ramp.getDataClip();
         float scaleAlpha = 1f - config.get( TRANSPARENCY_KEY ).floatValue();
         Shader shader = Shaders.fade( ramp.getShader(), scaleAlpha );
         return new GridStyle( xSizer, ySizer, xPhase, yPhase,
-                              scaling, shader, combiner );
+                              scaling, dataclip, shader, combiner );
     }
 
     public PlotLayer createLayer( DataGeom geom, DataSpec dataSpec,
@@ -471,6 +473,7 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
         private final double xPhase_;
         private final double yPhase_;
         private final Scaling scaling_;
+        private final Subrange dataclip_;
         private final Shader shader_;
         private final Combiner combiner_;
 
@@ -483,17 +486,20 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
          * @param  yPhase  Y axis bin reference point, 0..1
          * @param  scaling   scaling function for mapping densities to
          *                   colour map entries
+         * @param  dataclip  scaling input adjustment subrange
          * @param  shader   colour map
          * @param  combiner  value combination mode for bin calculation
          */
         public GridStyle( BinSizer xSizer, BinSizer ySizer,
-                          double xPhase, double yPhase,
-                          Scaling scaling, Shader shader, Combiner combiner ) {
+                          double xPhase, double yPhase, Scaling scaling,
+                          Subrange dataclip, Shader shader,
+                          Combiner combiner ) {
             xSizer_ = xSizer;
             ySizer_ = ySizer;
             xPhase_ = xPhase;
             yPhase_ = yPhase;
             scaling_ = scaling;
+            dataclip_ = dataclip;
             shader_ = shader;
             combiner_ = combiner;
         }
@@ -510,6 +516,7 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
             code = 23 * code + Float.floatToIntBits( (float) xPhase_ );
             code = 23 * code + Float.floatToIntBits( (float) yPhase_ );
             code = 23 * code + scaling_.hashCode();
+            code = 23 * code + dataclip_.hashCode();
             code = 23 * code + shader_.hashCode();
             code = 23 * code + combiner_.hashCode();
             return code;
@@ -524,6 +531,7 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
                     && this.xPhase_ == other.xPhase_
                     && this.yPhase_ == other.yPhase_
                     && this.scaling_.equals( other.scaling_ )
+                    && this.dataclip_.equals( other.dataclip_ )
                     && this.shader_.equals( other.shader_ )
                     && this.combiner_.equals( other.combiner_ );
             }
@@ -559,10 +567,10 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
         }
 
         public Drawing createDrawing( Surface surface,
-                                      Map<AuxScale,Range> auxRanges,
+                                      Map<AuxScale,Span> auxSpans,
                                       PaperType ptype ) {
             return new GridDrawing( (PlanarSurface) surface,
-                                    auxRanges.get( SCALE ), ptype );
+                                    auxSpans.get( SCALE ), ptype );
         }
 
         public Map<AuxScale,AuxReader> getAuxRangers() {
@@ -574,9 +582,12 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
                 public ValueInfo getAxisInfo( DataSpec dataSpec ) {
                     return getCombinedInfo( dataSpec );
                 }
+                public Scaling getScaling() {
+                    return gstyle_.scaling_;
+                }
                 public void adjustAuxRange( Surface surface, DataSpec dataSpec,
                                             DataStore dataStore, Object[] plans,
-                                            Range range ) {
+                                            Ranger ranger ) {
 
                     /* Work out the grid we need to sample over. */
                     PlanarSurface psurf = (PlanarSurface) surface;
@@ -599,7 +610,7 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
 
                     /* Use the grid pixer and binResult got by either means
                      * to adjust the aux range. */
-                    extendRange( range, psurf, pixer, binResult );
+                    extendRange( ranger, psurf, pixer, binResult );
                 }
             } );
             return map;
@@ -722,15 +733,15 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
         }
 
         /**
-         * Extends a given range object to accomodate values in a supplied
+         * Updates a given ranger object to accomodate values in a supplied
          * data grid.
          *
-         * @param   range  range object to adjust
+         * @param   ranger  ranger object to update
          * @param   surface   plot surface over which data is to be surveyed
          * @param   pixer    defines grid geometry
          * @param   binResult   accumulated grid data
          */
-        private void extendRange( Range range, PlanarSurface surface,
+        private void extendRange( Ranger ranger, PlanarSurface surface,
                                   GridPixer pixer, BinList.Result binResult ) {
             double[][] dlims = surface.getDataLimits();
             double binFactor =
@@ -745,7 +756,8 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
                 for ( int ix = ixlo; ix <= ixhi; ix++ ) {
                     int ibin = pixer.getBinIndex( ix, iy );
                     assert ibin >= 0;
-                    range.submit( binFactor * binResult.getBinValue( ibin ) );
+                    ranger.submitDatum( binFactor
+                                      * binResult.getBinValue( ibin ) );
                 }
             }
         }
@@ -780,20 +792,20 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
         private class GridDrawing implements Drawing {
 
             private final PlanarSurface surface_;
-            private final Range auxRange_;
+            private final Span auxSpan_;
             private final PaperType ptype_;
 
             /**
              * Constructor.
              *
              * @param   surface   plotting surface
-             * @param   auxRange  range defining colour scaling
+             * @param   auxSpan  range defining colour scaling
              * @param   paperType  paper type
              */
-            GridDrawing( PlanarSurface surface, Range auxRange,
+            GridDrawing( PlanarSurface surface, Span auxSpan,
                          PaperType ptype ) {
                 surface_ = surface;
-                auxRange_ = auxRange;
+                auxSpan_ = auxSpan;
                 ptype_ = ptype;
             }
 
@@ -822,8 +834,8 @@ public class GridPlotter implements Plotter<GridPlotter.GridStyle> {
                 ptype_.placeDecal( paper, new Decal() {
                     public void paintDecal( Graphics g ) {
                         Scaler scaler =
-                            Scaling.createRangeScaler( gstyle_.scaling_,
-                                                       auxRange_ );
+                            auxSpan_.createScaler( gstyle_.scaling_,
+                                                   gstyle_.dataclip_ );
                         IndexColorModel colorModel =
                             PixelImage.createColorModel( gstyle_.shader_,
                                                          true );
