@@ -12,10 +12,13 @@ import java.util.Map;
 import java.util.logging.Logger;
 import javax.swing.Icon;
 import uk.ac.starlink.table.DefaultValueInfo;
+import uk.ac.starlink.table.DescribedValue;
+import uk.ac.starlink.table.HealpixTableInfo;
+import uk.ac.starlink.table.StarTable;
 import uk.ac.starlink.table.ValueInfo;
+import uk.ac.starlink.ttools.cone.HealpixTiling;
 import uk.ac.starlink.ttools.func.Tilings;
 import uk.ac.starlink.ttools.gui.ResourceIcon;
-import uk.ac.starlink.ttools.plot.Range;
 import uk.ac.starlink.ttools.plot.Shader;
 import uk.ac.starlink.ttools.plot.Shaders;
 import uk.ac.starlink.ttools.plot.Style;
@@ -27,9 +30,12 @@ import uk.ac.starlink.ttools.plot2.Drawing;
 import uk.ac.starlink.ttools.plot2.LayerOpt;
 import uk.ac.starlink.ttools.plot2.PlotLayer;
 import uk.ac.starlink.ttools.plot2.PlotUtil;
+import uk.ac.starlink.ttools.plot2.Ranger;
 import uk.ac.starlink.ttools.plot2.ReportMap;
 import uk.ac.starlink.ttools.plot2.Scaler;
 import uk.ac.starlink.ttools.plot2.Scaling;
+import uk.ac.starlink.ttools.plot2.Span;
+import uk.ac.starlink.ttools.plot2.Subrange;
 import uk.ac.starlink.ttools.plot2.Surface;
 import uk.ac.starlink.ttools.plot2.config.ComboBoxSpecifier;
 import uk.ac.starlink.ttools.plot2.config.ConfigKey;
@@ -47,9 +53,9 @@ import uk.ac.starlink.ttools.plot2.data.DataSpec;
 import uk.ac.starlink.ttools.plot2.data.DataStore;
 import uk.ac.starlink.ttools.plot2.data.FloatingCoord;
 import uk.ac.starlink.ttools.plot2.data.InputMeta;
-import uk.ac.starlink.ttools.plot2.data.IntegerCoord;
 import uk.ac.starlink.ttools.plot2.data.Tuple;
 import uk.ac.starlink.ttools.plot2.data.TupleSequence;
+import uk.ac.starlink.ttools.plot2.geom.HealpixDataGeom;
 import uk.ac.starlink.ttools.plot2.geom.Rotation;
 import uk.ac.starlink.ttools.plot2.geom.SkySurface;
 import uk.ac.starlink.ttools.plot2.geom.SkySurfaceFactory;
@@ -71,28 +77,11 @@ public class HealpixPlotter
     private final int icHealpix_;
     private final int icValue_;
 
-    /** Maximum HEALPix level supported by this plotter. */
-    /* 13 is the largest value for which 32-bit pixel indices will suffice.
-     * Getting round that is not so hard, but some parts of the
-     * implementation are not efficient for finer grids, so other
-     * implementation work would have to be done to increase this value.
-     * In fact even at 13 performance is pretty poor at present. */
-    public static final int MAX_LEVEL = 13;
+    /** Currently always works with HEALPix NESTED scheme. */
+    public static final boolean IS_NEST = true;
 
-    /** Coordinate for HEALPix index. */
-    public static final IntegerCoord HEALPIX_COORD =
-        new IntegerCoord(
-            new InputMeta( "healpix", "HEALPix index" )
-           .setShortDescription( "HEALPix index" )
-           .setXmlDescription( new String[] {
-                "<p>HEALPix index indicating the sky position of the tile",
-                "whose value is plotted.",
-                "If not supplied, the assumption is that the supplied table",
-                "contains one row for each HEALPix tile at a given level,",
-                "in ascending order.",
-                "</p>",
-            } )
-        , false, IntegerCoord.IntType.INT );
+    /** Maximum HEALPix level supported by this plotter. */
+    public static final int MAX_LEVEL = HealpixTiling.MAX_LEVEL;
 
     /** Coordinate for value determining tile colours. */
     public static final FloatingCoord VALUE_COORD =
@@ -200,8 +189,7 @@ public class HealpixPlotter
      */
     public HealpixPlotter( boolean transparent ) {
         super( "Healpix", ResourceIcon.PLOT_HEALPIX,
-               CoordGroup.createCoordGroup( 0, new Coord[] { HEALPIX_COORD,
-                                                             VALUE_COORD } ),
+               CoordGroup.createCoordGroup( 1, new Coord[] { VALUE_COORD } ),
                false );
         icHealpix_ = 0;
         icValue_ = 1;
@@ -232,8 +220,8 @@ public class HealpixPlotter
         return sbuf.toString();
     }
 
-    public ConfigKey[] getStyleKeys() {
-        List<ConfigKey> keyList = new ArrayList<ConfigKey>();
+    public ConfigKey<?>[] getStyleKeys() {
+        List<ConfigKey<?>> keyList = new ArrayList<ConfigKey<?>>();
         keyList.add( DATALEVEL_KEY );
         keyList.add( DATASYS_KEY );
         keyList.add( VIEWSYS_KEY );
@@ -246,7 +234,7 @@ public class HealpixPlotter
         if ( reportAuxKeys_ ) {
             keyList.addAll( Arrays.asList( RAMP_KEYS.getKeys() ) );
         }
-        return keyList.toArray( new ConfigKey[ 0 ] );
+        return keyList.toArray( new ConfigKey<?>[ 0 ] );
     }
 
     public HealpixStyle createStyle( ConfigMap config ) {
@@ -259,18 +247,36 @@ public class HealpixPlotter
         SolidAngleUnit angle = config.get( ANGLE_KEY );
         Rotation rotation = Rotation.createRotation( dataSys, viewSys );
         Scaling scaling = ramp.getScaling();
+        Subrange dataclip = ramp.getDataClip();
         float scaleAlpha = 1f - config.get( TRANSPARENCY_KEY ).floatValue();
         Shader shader = Shaders.fade( ramp.getShader(), scaleAlpha );
-        return new HealpixStyle( dataLevel, degrade, rotation, scaling, shader,
-                                 combiner, angle );
+        return new HealpixStyle( dataLevel, degrade, rotation, scaling,
+                                 dataclip, shader, combiner, angle );
     }
 
-    public PlotLayer createLayer( DataGeom geom, DataSpec dataSpec,
+    public PlotLayer createLayer( DataGeom geom, final DataSpec dataSpec,
                                   HealpixStyle style ) {
-        int dataLevel = style.dataLevel_ >= 0
-                  ? style.dataLevel_
-                  : guessDataLevel( dataSpec.getSourceTable().getRowCount() );
+        if ( ! (geom instanceof HealpixDataGeom) ) {
+            throw new IllegalArgumentException( "DataGeom not Healpix: "
+                                              + geom );
+        }
+        StarTable table = dataSpec.getSourceTable();
+        List<DescribedValue> tparams = table.getParameters();
+        HealpixTableInfo hpxInfo = HealpixTableInfo.isHealpix( tparams )
+                                 ? HealpixTableInfo.fromParams( tparams )
+                                 : null;
+        int dataLevel = -1;
+        if ( dataLevel < 0 && style.dataLevel_ >= 0 ) {
+            dataLevel = style.dataLevel_;
+        }
+        if ( dataLevel < 0 && hpxInfo != null ) {
+            dataLevel = hpxInfo.getLevel();
+        }
+        if ( dataLevel < 0 ) {
+            dataLevel = guessDataLevel( table.getRowCount() );
+        }
         if ( dataLevel >= 0 ) {
+            final int nside = 1 << dataLevel;
             IndexReader rdr =
                   dataSpec.isCoordBlank( icHealpix_ )
                 ? new IndexReader() {
@@ -280,8 +286,8 @@ public class HealpixPlotter
                   }
                 : new IndexReader() {
                       public long getHealpixIndex( Tuple tuple ) {
-                          return HEALPIX_COORD.readIntCoord( tuple,
-                                                             icHealpix_ );
+                          return HealpixDataGeom.HEALPIX_COORD
+                                .readLongCoord( tuple, icHealpix_ );
                       }
                   };
             return new BinsHealpixLayer( geom, dataSpec, style,
@@ -349,7 +355,8 @@ public class HealpixPlotter
         }
         ConfigKey<Integer> key = new IntegerConfigKey( meta, -1 ) {
             public Specifier<Integer> createSpecifier() {
-                return new ComboBoxSpecifier( levelOptions );
+                return new ComboBoxSpecifier<Integer>( Integer.class,
+                                                       levelOptions );
             }
         };
         return key;
@@ -363,6 +370,7 @@ public class HealpixPlotter
         private final int degrade_;
         private final Rotation rotation_;
         private final Scaling scaling_;
+        private final Subrange dataclip_;
         private final Shader shader_;
         private final Combiner combiner_;
         private final SolidAngleUnit angle_;
@@ -377,17 +385,19 @@ public class HealpixPlotter
          * @param   rotation  sky rotation to be applied before plotting
          * @param   scaling   scaling function for mapping densities to
          *                    colour map entries
+         * @param   dataclip  scaling input range adjustment
          * @param   shader   colour map
          * @param   combiner  combiner, only relevant if degrade is non-zero
          * @param   angle      solid angle configuration for scaling
          */
         public HealpixStyle( int dataLevel, int degrade, Rotation rotation,
-                             Scaling scaling, Shader shader,
+                             Scaling scaling, Subrange dataclip, Shader shader,
                              Combiner combiner, SolidAngleUnit angle ) {
             dataLevel_ = dataLevel;
             degrade_ = degrade;
             rotation_ = rotation;
             scaling_ = scaling;
+            dataclip_ = dataclip;
             shader_ = shader;
             combiner_ = combiner;
             angle_ = angle;
@@ -416,6 +426,7 @@ public class HealpixPlotter
             code = 23 * code + degrade_;
             code = 23 * code + rotation_.hashCode();
             code = 23 * code + scaling_.hashCode();
+            code = 23 * code + dataclip_.hashCode();
             code = 23 * code + shader_.hashCode();
             code = 23 * code + combiner_.hashCode();
             code = 23 * code + angle_.hashCode();
@@ -430,6 +441,7 @@ public class HealpixPlotter
                     && this.degrade_ == other.degrade_
                     && this.rotation_.equals( other.rotation_ )
                     && this.scaling_.equals( other.scaling_ )
+                    && this.dataclip_.equals( other.dataclip_ )
                     && this.shader_.equals( other.shader_ )
                     && this.combiner_.equals( other.combiner_ )
                     && this.angle_.equals( other.angle_ );
@@ -471,6 +483,7 @@ public class HealpixPlotter
             assert hstyle.degrade_ >= 0;
         }
 
+        @Override
         public Map<AuxScale,AuxReader> getAuxRangers() {
             Map<AuxScale,AuxReader> map = new HashMap<AuxScale,AuxReader>();
             map.put( SCALE, new AuxReader() {
@@ -480,29 +493,33 @@ public class HealpixPlotter
                 public ValueInfo getAxisInfo( DataSpec dataSpec ) {
                     return getCombinedInfo( dataSpec );
                 }
+                public Scaling getScaling() {
+                    return hstyle_.scaling_;
+                }
                 public void adjustAuxRange( Surface surface, DataSpec dataSpec,
                                             DataStore dataStore,
-                                            Object[] knownPlans, Range range ) {
+                                            Object[] knownPlans,
+                                            Ranger ranger ) {
                     TilePlan tplan = getTilePlan( knownPlans );
                     BinList.Result binResult =
                         tplan == null ? readBins( dataSpec, dataStore )
                                       : tplan.binResult_;
                     createTileRenderer( surface )
-                   .extendAuxRange( range, binResult );
+                   .extendAuxRange( ranger, binResult );
                 }
             } );
             return map;
         }
 
         public Drawing createDrawing( Surface surf,
-                                      Map<AuxScale,Range> auxRanges,
+                                      Map<AuxScale,Span> auxSpans,
                                       final PaperType paperType ) {
             final DataSpec dataSpec = getDataSpec();
             final Combiner combiner = hstyle_.combiner_;
             final Shader shader = hstyle_.shader_;
             final Scaler scaler =
-                Scaling.createRangeScaler( hstyle_.scaling_,
-                                           auxRanges.get( SCALE ) );
+                auxSpans.get( SCALE )
+                        .createScaler( hstyle_.scaling_, hstyle_.dataclip_ );
             final SkyTileRenderer renderer = createTileRenderer( surf );
             return new Drawing() {
                 public Object calculatePlan( Object[] knownPlans,
@@ -591,13 +608,7 @@ public class HealpixPlotter
             int shift = degrade * 2;
             long nbin = 12 * ( 1L << ( 2 * viewLevel_ ) );
             Combiner combiner = hstyle_.combiner_;
-            BinList binList = null;
-            if ( nbin < 1e6 ) {
-                binList = combiner.createArrayBinList( (int) nbin );
-            }
-            if ( binList == null ) {
-                binList = combiner.createHashBinList( nbin );
-            }
+            BinList binList = Combiner.createDefaultBinList( combiner, nbin );
             TupleSequence tseq = dataStore.getTupleSequence( dataSpec );
             while ( tseq.next() ) {
                 double value = tseq.getDoubleValue( icValue_ );
