@@ -32,6 +32,7 @@ import uk.ac.starlink.ttools.plot2.PlotUtil;
 import uk.ac.starlink.ttools.plot2.ReportKey;
 import uk.ac.starlink.ttools.plot2.ReportMap;
 import uk.ac.starlink.ttools.plot2.ReportMeta;
+import uk.ac.starlink.ttools.plot2.Span;
 import uk.ac.starlink.ttools.plot2.Surface;
 import uk.ac.starlink.ttools.plot2.config.BooleanConfigKey;
 import uk.ac.starlink.ttools.plot2.config.ConfigKey;
@@ -234,9 +235,9 @@ public class HistogramPlotter
         return histoCoordGrp_;
     }
 
-    public ConfigKey[] getStyleKeys() {
-        List<ConfigKey> list = new ArrayList<ConfigKey>();
-        list.addAll( Arrays.asList( new ConfigKey[] {
+    public ConfigKey<?>[] getStyleKeys() {
+        List<ConfigKey<?>> list = new ArrayList<ConfigKey<?>>();
+        list.addAll( Arrays.asList( new ConfigKey<?>[] {
             StyleKeys.COLOR,
             StyleKeys.TRANSPARENCY,
             BINSIZER_KEY,
@@ -246,14 +247,14 @@ public class HistogramPlotter
         if ( unitKey_ != null ) {
             list.add( unitKey_ );
         }
-        list.addAll( Arrays.asList( new ConfigKey[] {
+        list.addAll( Arrays.asList( new ConfigKey<?>[] {
             StyleKeys.CUMULATIVE,
             StyleKeys.NORMALISE,
             StyleKeys.BAR_FORM,
             THICK_KEY,
             StyleKeys.DASH,
         } ) );
-        return list.toArray( new ConfigKey[ 0 ] );
+        return list.toArray( new ConfigKey<?>[ 0 ] );
     }
 
     public HistoStyle createStyle( ConfigMap config ) {
@@ -302,7 +303,7 @@ public class HistogramPlotter
             return new AbstractPlotLayer( this, histoDataGeom_, dataSpec,
                                           style, layerOpt ) {
                 public Drawing createDrawing( Surface surface,
-                                              Map<AuxScale,Range> auxRanges,
+                                              Map<AuxScale,Span> auxSpans,
                                               final PaperType paperType ) {
                     if ( ! ( surface instanceof PlanarSurface ) ) {
                         throw new IllegalArgumentException( "Not planar surface"
@@ -339,7 +340,7 @@ public class HistogramPlotter
                             }
                             BinBag binBag =
                                 readBins( xlog, binWidth, binPhase, combiner,
-                                          xlo, dataSpec, dataStore );
+                                          xlo, xhi, dataSpec, dataStore );
                             return new HistoPlan( binBag, dataSpec );
                         }
                         public void paintData( Object plan, Paper paper,
@@ -398,7 +399,7 @@ public class HistogramPlotter
                     double binWidth = sizer.getWidth( xlog, xlo, xhi, xround );
                     BinBag binBag =
                         readBins( xlog, binWidth, binPhase, combiner,
-                                  xlo, dataSpec, dataStore );
+                                  xlo, xhi, dataSpec, dataStore );
 
                     /* Assume y=0 is always of interest for a histogram. */
                     yRange.submit( 0 );
@@ -445,13 +446,15 @@ public class HistogramPlotter
      * @param   binWidth  additive/multiplicative bin width
      * @param   binPhase   bin reference point, 0..1
      * @param   combiner   bin aggregation mode
-     * @param   point     representative data value along axis
+     * @param   xlo       representative lower value along axis
+     * @param   xhi       representative upper value along axis
      * @param   dataSpec  specification for histogram data values
      * @param   dataStore  data storage
      */
     private BinBag readBins( boolean xlog, double binWidth, double binPhase,
-                             Combiner combiner, double point, DataSpec dataSpec,
-                             DataStore dataStore ) {
+                             Combiner combiner, double xlo, double xhi,
+                             DataSpec dataSpec, DataStore dataStore ) {
+        double point = PlotUtil.scaleValue( xlo, xhi, 0.5, xlog );
         BinBag binBag = new BinBag( xlog, binWidth, binPhase, combiner, point );
         TupleSequence tseq = dataStore.getTupleSequence( dataSpec );
         if ( weightCoord_ == null || dataSpec.isCoordBlank( icWeight_ ) ) {
