@@ -10,7 +10,6 @@ import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 import uk.ac.starlink.ttools.gui.ResourceIcon;
-import uk.ac.starlink.ttools.plot.Range;
 import uk.ac.starlink.ttools.plot2.Anchor;
 import uk.ac.starlink.ttools.plot2.AuxScale;
 import uk.ac.starlink.ttools.plot2.Captioner;
@@ -23,6 +22,7 @@ import uk.ac.starlink.ttools.plot2.PlotLayer;
 import uk.ac.starlink.ttools.plot2.PlotUtil;
 import uk.ac.starlink.ttools.plot2.PointCloud;
 import uk.ac.starlink.ttools.plot2.ReportMap;
+import uk.ac.starlink.ttools.plot2.Span;
 import uk.ac.starlink.ttools.plot2.SubCloud;
 import uk.ac.starlink.ttools.plot2.Surface;
 import uk.ac.starlink.ttools.plot2.config.CaptionerKeySet;
@@ -131,16 +131,16 @@ public class LabelPlotter extends AbstractPlotter<LabelStyle> {
         } );
     }
 
-    public ConfigKey[] getStyleKeys() {
-        List<ConfigKey> list = new ArrayList<ConfigKey>();
+    public ConfigKey<?>[] getStyleKeys() {
+        List<ConfigKey<?>> list = new ArrayList<ConfigKey<?>>();
         list.addAll( Arrays.asList( CAPTIONER_KEYSET.getKeys() ) );
-        list.addAll( Arrays.asList( new ConfigKey[] {
+        list.addAll( Arrays.asList( new ConfigKey<?>[] {
             StyleKeys.ANCHOR,
             StyleKeys.COLOR,
             SPACING_KEY,
             CROWDLIMIT_KEY,
         } ) );
-        return list.toArray( new ConfigKey[ 0 ] );
+        return list.toArray( new ConfigKey<?>[ 0 ] );
     }
 
     public LabelStyle createStyle( ConfigMap config ) throws ConfigException {
@@ -164,7 +164,7 @@ public class LabelPlotter extends AbstractPlotter<LabelStyle> {
         LayerOpt opt = new LayerOpt( style.getColor(), true );
         return new AbstractPlotLayer( this, geom, dataSpec, style, opt ) {
             public Drawing createDrawing( Surface surface,
-                                          Map<AuxScale,Range> auxRanges,
+                                          Map<AuxScale,Span> auxSpans,
                                           PaperType paperType ) {
                 if ( paperType instanceof PaperType2D ) {
                     return new LabelDrawing2D( geom, dataSpec, style, surface,
@@ -246,9 +246,9 @@ public class LabelPlotter extends AbstractPlotter<LabelStyle> {
             for ( int i = 0; i < knownPlans.length; i++ ) {
                 Object plan = knownPlans[ i ];
                 if ( plan instanceof LabelPlan &&
-                    ((LabelPlan) plan).matches( geom_, dataSpec_, surface_,
-                                                spacing, crowdLimit,
-                                                clazz_ ) ) {
+                    ((LabelPlan<?>) plan).matches( geom_, dataSpec_, surface_,
+                                                   spacing, crowdLimit,
+                                                   clazz_ ) ) {
                     return plan;
                 }
             }
@@ -564,7 +564,7 @@ public class LabelPlotter extends AbstractPlotter<LabelStyle> {
          * @param  parameterising class
          */
         boolean matches( DataGeom geom, DataSpec dataSpec, Surface surface,
-                         int spacing, byte crowdLimit, Class clazz ) {
+                         int spacing, byte crowdLimit, Class<?> clazz ) {
             return geom.equals( geom_ )
                 && dataSpec.equals( dataSpec_ )
                 && surface.equals( surface_ )
