@@ -12,7 +12,6 @@ import java.util.Arrays;
 import java.util.List;
 import java.util.Map;
 import uk.ac.starlink.ttools.gui.ResourceIcon;
-import uk.ac.starlink.ttools.plot.Range;
 import uk.ac.starlink.ttools.plot2.AuxScale;
 import uk.ac.starlink.ttools.plot2.DataGeom;
 import uk.ac.starlink.ttools.plot2.Decal;
@@ -23,6 +22,7 @@ import uk.ac.starlink.ttools.plot2.PlotUtil;
 import uk.ac.starlink.ttools.plot2.ReportKey;
 import uk.ac.starlink.ttools.plot2.ReportMap;
 import uk.ac.starlink.ttools.plot2.ReportMeta;
+import uk.ac.starlink.ttools.plot2.Span;
 import uk.ac.starlink.ttools.plot2.Surface;
 import uk.ac.starlink.ttools.plot2.config.ConfigKey;
 import uk.ac.starlink.ttools.plot2.config.ConfigMap;
@@ -115,12 +115,12 @@ public class LinearFitPlotter extends AbstractPlotter<LineStyle> {
         } );
     }
 
-    public ConfigKey[] getStyleKeys() {
-        List<ConfigKey> list = new ArrayList<ConfigKey>();
+    public ConfigKey<?>[] getStyleKeys() {
+        List<ConfigKey<?>> list = new ArrayList<ConfigKey<?>>();
         list.add( StyleKeys.COLOR );
         list.addAll( Arrays.asList( StyleKeys.getStrokeKeys() ) );
         list.add( StyleKeys.ANTIALIAS );
-        return list.toArray( new ConfigKey[ 0 ] );
+        return list.toArray( new ConfigKey<?>[ 0 ] );
     }
 
     public LineStyle createStyle( ConfigMap config ) {
@@ -137,7 +137,7 @@ public class LinearFitPlotter extends AbstractPlotter<LineStyle> {
         final CoordGroup cgrp = getCoordGroup();
         return new AbstractPlotLayer( this, geom, dataSpec, style, layerOpt ) {
             public Drawing createDrawing( Surface surface,
-                                          Map<AuxScale,Range> auxRanges,
+                                          Map<AuxScale,Span> auxSpans,
                                           PaperType paperType ) {
                 return new LinearFitDrawing( (PlanarSurface) surface, geom,
                                              dataSpec, cgrp, style, paperType );
@@ -329,8 +329,9 @@ public class LinearFitPlotter extends AbstractPlotter<LineStyle> {
                  PlotUtil.isPointFinite( gp2 ) ) {
                 LineTracer tracer =
                     style.createLineTracer( g, bounds, 2, false ); 
-                tracer.addVertex( gp1.x, gp1.y );
-                tracer.addVertex( gp2.x, gp2.y );
+                Color color = style.getColor();
+                tracer.addVertex( gp1.x, gp1.y, color );
+                tracer.addVertex( gp2.x, gp2.y, color );
                 tracer.flush();
             }
         }
