@@ -18,6 +18,7 @@ import uk.ac.starlink.ttools.plot2.Plotter;
 import uk.ac.starlink.ttools.plot2.ReportKey;
 import uk.ac.starlink.ttools.plot2.ReportMap;
 import uk.ac.starlink.ttools.plot2.ReportMeta;
+import uk.ac.starlink.ttools.plot2.Span;
 import uk.ac.starlink.ttools.plot2.Surface;
 import uk.ac.starlink.ttools.plot2.config.ConfigKey;
 import uk.ac.starlink.ttools.plot2.config.ConfigMeta;
@@ -277,7 +278,7 @@ public abstract class Pixel1dPlotter<S extends Style> implements Plotter<S> {
         return new AbstractPlotLayer( this, pixoDataGeom_, dataSpec,
                                       style, layerOpt ) {
             public Drawing createDrawing( Surface surface,
-                                          Map<AuxScale,Range> auxRanges,
+                                          Map<AuxScale,Span> auxSpans,
                                           final PaperType paperType ) {
                 if ( ! ( surface instanceof PlanarSurface ) ) {
                     throw new IllegalArgumentException( "Not planar surface "
@@ -484,7 +485,7 @@ public abstract class Pixel1dPlotter<S extends Style> implements Plotter<S> {
      * This is basically the kernel's extent, but it may be limited to some
      * maximum for practical purposes.
      *
-     * @param  kernel
+     * @param  kernel  smoothing kernel
      * @return  effective extent
      */
     public static int getEffectiveExtent( Kernel1d kernel ) {
