@@ -19,7 +19,6 @@ import uk.ac.starlink.table.DefaultValueInfo;
 import uk.ac.starlink.table.ValueInfo;
 import uk.ac.starlink.ttools.func.Tilings;
 import uk.ac.starlink.ttools.gui.ResourceIcon;
-import uk.ac.starlink.ttools.plot.Range;
 import uk.ac.starlink.ttools.plot.Shader;
 import uk.ac.starlink.ttools.plot.Shaders;
 import uk.ac.starlink.ttools.plot.Style;
@@ -34,11 +33,15 @@ import uk.ac.starlink.ttools.plot2.LayerOpt;
 import uk.ac.starlink.ttools.plot2.Pixer;
 import uk.ac.starlink.ttools.plot2.PlotLayer;
 import uk.ac.starlink.ttools.plot2.PlotUtil;
+import uk.ac.starlink.ttools.plot2.Ranger;
 import uk.ac.starlink.ttools.plot2.ReportKey;
 import uk.ac.starlink.ttools.plot2.ReportMap;
 import uk.ac.starlink.ttools.plot2.ReportMeta;
 import uk.ac.starlink.ttools.plot2.Scaler;
 import uk.ac.starlink.ttools.plot2.Scaling;
+import uk.ac.starlink.ttools.plot2.Scalings;
+import uk.ac.starlink.ttools.plot2.Span;
+import uk.ac.starlink.ttools.plot2.Subrange;
 import uk.ac.starlink.ttools.plot2.Surface;
 import uk.ac.starlink.ttools.plot2.config.ConfigKey;
 import uk.ac.starlink.ttools.plot2.config.ConfigMap;
@@ -222,7 +225,7 @@ public abstract class ShapeMode implements ModePlotter.Mode {
      *
      * @return  array of config keys for mode
      */
-    public abstract ConfigKey[] getConfigKeys();
+    public abstract ConfigKey<?>[] getConfigKeys();
 
     /**
      * Returns an object which will do the work of colouring in shapes
@@ -295,7 +298,7 @@ public abstract class ShapeMode implements ModePlotter.Mode {
      * @param  key   report key for new entry
      * @param  axis   axis along which dimension is to be reported
      */
-    private static void addPixelSize( ReportMap report, ReportKey key,
+    private static void addPixelSize( ReportMap report, ReportKey<Double> key,
                                       Axis axis ) {
         if ( axis.isLinear() ) {
             int g0 = axis.getGraphicsLimits()[ 0 ];
@@ -353,13 +356,13 @@ public abstract class ShapeMode implements ModePlotter.Mode {
             }
         }
 
-        public ConfigKey[] getConfigKeys() {
-            List<ConfigKey> keyList = new ArrayList<ConfigKey>();
+        public ConfigKey<?>[] getConfigKeys() {
+            List<ConfigKey<?>> keyList = new ArrayList<ConfigKey<?>>();
             keyList.add( StyleKeys.COLOR );
             if ( transparent_ ) {
                 keyList.add( StyleKeys.OPAQUE );
             }
-            return keyList.toArray( new ConfigKey[ 0 ] );
+            return keyList.toArray( new ConfigKey<?>[ 0 ] );
         }
 
         public Stamper createStamper( ConfigMap config ) {
@@ -597,8 +600,8 @@ public abstract class ShapeMode implements ModePlotter.Mode {
             } );
         }
 
-        public ConfigKey[] getConfigKeys() {
-            return new ConfigKey[] {
+        public ConfigKey<?>[] getConfigKeys() {
+            return new ConfigKey<?>[] {
                 StyleKeys.COLOR,
                 StyleKeys.TRANSPARENT_LEVEL,
             };
@@ -771,15 +774,18 @@ public abstract class ShapeMode implements ModePlotter.Mode {
         public PlotLayer createLayer( ShapePlotter plotter, ShapeForm form,
                                       DataGeom geom, DataSpec dataSpec,
                                       Outliner outliner, Stamper stamper ) {
-            final Shader shader = ((DensityStamper) stamper).shader_;
-            final Scaling scaling = ((DensityStamper) stamper).scaling_;
+            DensityStamper dstamper = (DensityStamper) stamper;
+            final Shader shader = dstamper.shader_;
+            final Scaling scaling = dstamper.scaling_;
+            final Subrange dataclip = dstamper.dataclip_;
             ShapeStyle style = new ShapeStyle( outliner, stamper );
             LayerOpt opt = Shaders.isTransparent( shader ) ? LayerOpt.NO_SPECIAL
                                                            : LayerOpt.OPAQUE;
             return new ShapePlotLayer( plotter, geom, dataSpec, style, opt,
                                        outliner ) {
                 public Drawing createDrawing( DrawSpec drawSpec ) {
-                    return new DensityDrawing( drawSpec, shader, scaling );
+                    return new DensityDrawing( drawSpec, shader, scaling,
+                                               dataclip );
                 }
             };
         }
@@ -790,6 +796,7 @@ public abstract class ShapeMode implements ModePlotter.Mode {
         private static class DensityDrawing extends BinShapeDrawing {
             private final Shader shader_;
             private final Scaling scaling_;
+            private final Subrange dataclip_;
 
             /**
              * Constructor.
@@ -797,12 +804,14 @@ public abstract class ShapeMode implements ModePlotter.Mode {
              * @param  drawSpec  common drawing attributes
              * @param  shader  determines colours in colour map
              * @param  scaling  quantitative mapping from counts to colour
+             * @param  dataclip  count mapping range adjustment
              */
             DensityDrawing( DrawSpec drawSpec, Shader shader,
-                            Scaling scaling ) {
+                            Scaling scaling, Subrange dataclip ) {
                 super( drawSpec );
                 shader_ = shader;
                 scaling_ = scaling;
+                dataclip_ = dataclip;
             }
 
             PixelImage createPixelImage( Object plan ) {
@@ -829,26 +838,28 @@ public abstract class ShapeMode implements ModePlotter.Mode {
              * @param   nlevel   number of levels represented in output
              */
             private void scaleLevels( int[] buf, int nlevel ) {
+                Ranger ranger =
+                    Scalings.createRanger( new Scaling[] { scaling_ } );
                 int n = buf.length;
                 int max = 0;
                 for ( int i = 0; i < n; i++ ) {
-                    max = Math.max( max, buf[ i ] );
+                    int b = buf[ i ];
+                    if ( b > 0 ) {
+                        max = Math.max( b, max );
+                        ranger.submitDatum( b );
+                    }
                 }
 
                 /* Leave 0 as 0 - this is transparent (no plot).
                  * Values >= 1 map to range 1..nlevel. */
                 if ( max > 0 ) {
-                    max = Math.max( max, PlotUtil.MIN_RAMP_UNIT );
-                    CountScaler scaler =
-                        new CountScaler( scaling_, max, nlevel );
-                    if ( max == 1 && scaler.scaleCount( 1 ) == 1 ) {
-                        // special case: array is already in the required form
-                        // (zeros and ones), no action required
-                    }
-                    else {
-                        for ( int i = 0; i < n; i++ ) {
-                            buf[ i ] = scaler.scaleCount( buf[ i ] );
-                        }
+                    ranger.submitDatum( 1 );
+                    ranger.submitDatum( PlotUtil.MIN_RAMP_UNIT );
+                    Scaler scaler = ranger.createSpan()
+                                   .createScaler( scaling_, dataclip_ );
+                    CountScaler countScaler = new CountScaler( scaler, nlevel );
+                    for ( int i = 0; i < n; i++ ) {
+                        buf[ i ] = countScaler.scaleCount( buf[ i ] );
                     }
                 }
             }
@@ -861,16 +872,20 @@ public abstract class ShapeMode implements ModePlotter.Mode {
     public static class DensityStamper implements Stamper {
         final Shader shader_;
         final Scaling scaling_;
+        final Subrange dataclip_;
 
         /**
          * Constructor.
          *
          * @param   shader  colour shader
          * @param  scaling  count scaling strategy
+         * @param  dataclip  scaling range adjustment
          */
-        public DensityStamper( Shader shader, Scaling scaling ) {
+        public DensityStamper( Shader shader, Scaling scaling,
+                               Subrange dataclip ) {
             shader_ = shader;
             scaling_ = scaling;
+            dataclip_ = dataclip;
         }
 
         public Icon createLegendIcon( Outliner outliner ) {
@@ -883,7 +898,8 @@ public abstract class ShapeMode implements ModePlotter.Mode {
             if ( o instanceof DensityStamper ) {
                 DensityStamper other = (DensityStamper) o;
                 return this.shader_.equals( other.shader_ )
-                    && this.scaling_.equals( other.scaling_ );
+                    && this.scaling_.equals( other.scaling_ )
+                    && this.dataclip_.equals( other.dataclip_ );
             }
             else {
                 return false;
@@ -895,6 +911,7 @@ public abstract class ShapeMode implements ModePlotter.Mode {
             int code = 3311;
             code = 23 * code + shader_.hashCode();
             code = 23 * code + scaling_.hashCode();
+            code = 23 * code + dataclip_.hashCode();
             return code;
         }
     }
@@ -934,8 +951,8 @@ public abstract class ShapeMode implements ModePlotter.Mode {
             } );
         }
 
-        public ConfigKey[] getConfigKeys() {
-            return new ConfigKey[] {
+        public ConfigKey<?>[] getConfigKeys() {
+            return new ConfigKey<?>[] {
                 StyleKeys.COLOR,
             };
         }
@@ -945,7 +962,8 @@ public abstract class ShapeMode implements ModePlotter.Mode {
             Shader baseShader = Shaders.stretch( Shaders.SCALE_V, 1f, 0.2f );
             Shader densityShader =
                 Shaders.applyShader( baseShader, baseColor, COLOR_MAP_SIZE );
-            return new DensityStamper( densityShader, Scaling.AUTO );
+            return new DensityStamper( densityShader, Scaling.AUTO,
+                                       new Subrange() );
         }
     }
 
@@ -984,11 +1002,11 @@ public abstract class ShapeMode implements ModePlotter.Mode {
             } );
         }
 
-        public ConfigKey[] getConfigKeys() {
-            List<ConfigKey> keyList = new ArrayList<ConfigKey>();
+        public ConfigKey<?>[] getConfigKeys() {
+            List<ConfigKey<?>> keyList = new ArrayList<ConfigKey<?>>();
             keyList.add( StyleKeys.COLOR );
             keyList.addAll( Arrays.asList( RAMP_KEYS.getKeys() ) );
-            return keyList.toArray( new ConfigKey[ 0 ] );
+            return keyList.toArray( new ConfigKey<?>[ 0 ] );
         }
 
         public Stamper createStamper( ConfigMap config ) {
@@ -998,7 +1016,8 @@ public abstract class ShapeMode implements ModePlotter.Mode {
             Shader densityShader =
                 Shaders.applyShader( baseShader, baseColor, COLOR_MAP_SIZE );
             Scaling scaling = ramp.getScaling();
-            return new DensityStamper( densityShader, scaling );
+            Subrange dataclip = ramp.getDataClip();
+            return new DensityStamper( densityShader, scaling, dataclip );
         }
     }
 
@@ -1061,8 +1080,8 @@ public abstract class ShapeMode implements ModePlotter.Mode {
             return sbuf.toString();
         }
 
-        public ConfigKey[] getConfigKeys() {
-            List<ConfigKey> list = new ArrayList<ConfigKey>();
+        public ConfigKey<?>[] getConfigKeys() {
+            List<ConfigKey<?>> list = new ArrayList<ConfigKey<?>>();
             if ( reportAuxKeys_ ) {
                 list.addAll( Arrays.asList( RAMP_KEYS.getKeys() ) );
             }
@@ -1070,20 +1089,21 @@ public abstract class ShapeMode implements ModePlotter.Mode {
             if ( transparent_ ) {
                 list.add( StyleKeys.AUX_OPAQUE );
             }
-            return list.toArray( new ConfigKey[ 0 ] );
+            return list.toArray( new ConfigKey<?>[ 0 ] );
         }
 
         public Stamper createStamper( ConfigMap config ) {
             RampKeySet.Ramp ramp = RAMP_KEYS.createValue( config );
             Shader shader = ramp.getShader();
             Scaling scaling = ramp.getScaling();
+            Subrange dataclip = ramp.getDataClip();
             Color nullColor = config.get( StyleKeys.AUX_NULLCOLOR );
             double opaque = transparent_
                           ? config.get( StyleKeys.AUX_OPAQUE )
                           : 1;
             float scaleAlpha = 1f / (float) opaque;
             Color baseColor = config.get( StyleKeys.COLOR );
-            return new ShadeStamper( shader, scaling, baseColor,
+            return new ShadeStamper( shader, scaling, dataclip, baseColor,
                                      nullColor, scaleAlpha );
         }
 
@@ -1098,6 +1118,7 @@ public abstract class ShapeMode implements ModePlotter.Mode {
             ShadeStamper shStamper = (ShadeStamper) stamper;
             final Shader shader = shStamper.shader_;
             final Scaling scaling = shStamper.scaling_;
+            final Subrange dataclip = shStamper.dataclip_;
             final Color baseColor = shStamper.baseColor_;
             final Color nullColor = shStamper.nullColor_;
             final float scaleAlpha = shStamper.scaleAlpha_;
@@ -1112,20 +1133,19 @@ public abstract class ShapeMode implements ModePlotter.Mode {
                     Map<AuxScale,AuxReader> map = super.getAuxRangers();
                     AuxReader shadeReader =
                         new FloatingCoordAuxReader( SHADE_COORD, iShadeCoord,
-                                                    geom, true );
+                                                    geom, true, scaling );
                     map.put( SCALE, shadeReader );
                     map.putAll( outliner.getAuxRangers( geom ) );
                     return map;
                 }
                 public Drawing createDrawing( Surface surface,
-                                              Map<AuxScale,Range> auxRanges,
+                                              Map<AuxScale,Span> auxSpans,
                                               PaperType paperType ) {
-                    Range shadeRange = auxRanges.get( SCALE );
-                    Scaler scaler =
-                        Scaling.createRangeScaler( scaling, shadeRange );
+                    Span shadeSpan = auxSpans.get( SCALE );
+                    Scaler scaler = shadeSpan.createScaler( scaling, dataclip );
                     DrawSpec drawSpec =
                         new DrawSpec( surface, geom, dataSpec, outliner,
-                                      auxRanges, paperType );
+                                      auxSpans, paperType );
 
                     // A possible optimisation would be in the case that we
                     // have an opaque 2D plot to return a planned drawing,
@@ -1205,14 +1225,14 @@ public abstract class ShapeMode implements ModePlotter.Mode {
             return sbuf.toString();
         }
 
-        public ConfigKey[] getConfigKeys() {
-            List<ConfigKey> keyList = new ArrayList<ConfigKey>();
+        public ConfigKey<?>[] getConfigKeys() {
+            List<ConfigKey<?>> keyList = new ArrayList<ConfigKey<?>>();
             keyList.add( StyleKeys.COLOR );
             keyList.add( COMBINER_KEY );
             if ( reportAuxKeys_ ) {
                 keyList.addAll( Arrays.asList( RAMP_KEYS.getKeys() ) );
             }
-            return keyList.toArray( new ConfigKey[ 0 ] );
+            return keyList.toArray( new ConfigKey<?>[ 0 ] );
         }
 
         public Stamper createStamper( ConfigMap config ) {
@@ -1222,8 +1242,9 @@ public abstract class ShapeMode implements ModePlotter.Mode {
             Shader shader =
                 Shaders.applyShader( baseShader, baseColor, COLOR_MAP_SIZE );
             Scaling scaling = ramp.getScaling();
+            Subrange dataclip = ramp.getDataClip();
             Combiner combiner = config.get( COMBINER_KEY );
-            return new WeightStamper( shader, scaling, combiner );
+            return new WeightStamper( shader, scaling, dataclip, combiner );
         }
 
         public PlotLayer createLayer( ShapePlotter plotter, ShapeForm form,
@@ -1336,6 +1357,9 @@ public abstract class ShapeMode implements ModePlotter.Mode {
                     public int getCoordIndex() {
                         return icWeight_;
                     }
+                    public Scaling getScaling() {
+                        return wstamper_.scaling_;
+                    }
                     public ValueInfo getAxisInfo( DataSpec dataSpec ) {
                         Combiner combiner = wstamper_.combiner_;
                         final ValueInfo weightInfo;
@@ -1361,16 +1385,16 @@ public abstract class ShapeMode implements ModePlotter.Mode {
                                                 DataSpec dataSpec,
                                                 DataStore dataStore,
                                                 Object[] knownPlans,
-                                                Range range ) {
+                                                Ranger ranger ) {
                         WeightPlan wplan =
                             getWeightPlan( knownPlans, surface, dataSpec );
                         final BinList.Result binResult;
                         if ( wplan == null ) {
-                            // no auxRanges - have to fake it.
-                            Map<AuxScale,Range> auxRanges =
-                                new HashMap<AuxScale,Range>();
+                            // no auxSpans - have to fake it.
+                            Map<AuxScale,Span> auxSpans =
+                                new HashMap<AuxScale,Span>();
                             binResult = readBinList( surface, dataSpec,
-                                                     dataStore, auxRanges )
+                                                     dataStore, auxSpans )
                                        .getResult();
                         }
                         else {
@@ -1380,7 +1404,7 @@ public abstract class ShapeMode implements ModePlotter.Mode {
                         for ( Iterator<Long> it = binResult.indexIterator();
                               it.hasNext(); ) {
                             long ibin = it.next().longValue();
-                            range.submit( binResult.getBinValue( ibin ) );
+                            ranger.submitDatum( binResult.getBinValue( ibin ) );
                         }
                     }
                 } );
@@ -1388,9 +1412,9 @@ public abstract class ShapeMode implements ModePlotter.Mode {
             }
 
             public Drawing createDrawing( Surface surface,
-                                          Map<AuxScale,Range> auxRanges,
+                                          Map<AuxScale,Span> auxSpans,
                                           PaperType paperType ) {
-                return new WeightDrawing( surface, auxRanges, paperType );
+                return new WeightDrawing( surface, auxSpans, paperType );
             }
 
             /**
@@ -1402,19 +1426,23 @@ public abstract class ShapeMode implements ModePlotter.Mode {
              * @param  dataSpec  data specification
              * @param  dataStore  data storage
              * @param  tseq    point sequence
-             * @param  auxRanges   aux data range map
+             * @param  auxSpans   aux data range map
              * @return  bin list that does not require scaling
              */
             private BinList readBinList( Surface surface, DataSpec dataSpec,
                                          DataStore dataStore,
-                                         Map<AuxScale,Range> auxRanges ) {
+                                         Map<AuxScale,Span> auxSpans ) {
                 DataGeom geom = getDataGeom();
                 WeightPaper wpaper =
                     new WeightPaper( surface.getPlotBounds(),
                                      wstamper_.combiner_ );
+                GlyphPaper.GlyphPaperType ptype = wpaper.getPaperType();
                 ShapePainter painter =
-                    outliner_.create2DPainter( surface, geom, auxRanges,
-                                               wpaper.getPaperType() );
+                      surface instanceof CubeSurface
+                    ? outliner_.create3DPainter( (CubeSurface) surface, geom,
+                                                 auxSpans, ptype )
+                    : outliner_.create2DPainter( surface, geom,
+                                                 auxSpans, ptype );
                 TupleSequence tseq = dataStore.getTupleSequence( dataSpec );
 
                 /* Under normal circumstances, use the submitted combiner
@@ -1462,20 +1490,20 @@ public abstract class ShapeMode implements ModePlotter.Mode {
              */
             private class WeightDrawing implements Drawing {
                 final Surface surface_;
-                final Map<AuxScale,Range> auxRanges_;
+                final Map<AuxScale,Span> auxSpans_;
                 final PaperType paperType_;
 
                 /**
                  * Constructor.
                  *
                  * @param  surface  plot surface
-                 * @param  auxRanges   aux data ranges
+                 * @param  auxSpans   aux data ranges
                  * @param  paperType   paper type
                  */
-                WeightDrawing( Surface surface, Map<AuxScale,Range> auxRanges,
+                WeightDrawing( Surface surface, Map<AuxScale,Span> auxSpans,
                                PaperType paperType ) {
                     surface_ = surface;
-                    auxRanges_ = auxRanges;
+                    auxSpans_ = auxSpans;
                     paperType_ = paperType;
                 }
 
@@ -1489,7 +1517,7 @@ public abstract class ShapeMode implements ModePlotter.Mode {
                     }
                     else {
                         BinList binList = readBinList( surface_, dataSpec,
-                                                       dataStore, auxRanges_ );
+                                                       dataStore, auxSpans_ );
                         int nbin = (int) binList.getSize();  // pixel count
                         Combiner combiner = binList.getCombiner();
                         BinList.Result binResult =
@@ -1530,16 +1558,15 @@ public abstract class ShapeMode implements ModePlotter.Mode {
                  */
                 private void paintBins( Graphics g, int nbin,
                                         BinList.Result binResult ) {
-                    Range auxRange = auxRanges_.get( SCALE );
-                    if ( auxRange == null ) {
-                        auxRange = new Range();
+                    Span auxSpan = auxSpans_.get( SCALE );
+                    if ( auxSpan == null ) {
+                        auxSpan = PlotUtil.EMPTY_SPAN;
                     }
                     Rectangle plotBounds = surface_.getPlotBounds();
                     IndexColorModel colorModel =
                         PixelImage.createColorModel( wstamper_.shader_, true );
-                    Scaler scaler =
-                        Scaling.createRangeScaler( wstamper_.scaling_,
-                                                   auxRange );
+                    Scaler scaler = auxSpan.createScaler( wstamper_.scaling_,
+                                                          wstamper_.dataclip_ );
                     int[] pixels =
                         scaleLevels( nbin, binResult, scaler,
                                      colorModel.getMapSize() - 1 );
@@ -1688,7 +1715,7 @@ public abstract class ShapeMode implements ModePlotter.Mode {
                 int nbin = gridder_.getLength();
                 BinList binlist = combiner.createArrayBinList( nbin );
                 binList_ = binlist == null
-                         ? combiner.createHashBinList( nbin )
+                         ? new HashBinList( nbin, combiner )
                          : binlist;
                 xoff_ = bounds_.x;
                 yoff_ = bounds_.y;
@@ -1722,6 +1749,7 @@ public abstract class ShapeMode implements ModePlotter.Mode {
     public static class WeightStamper implements Stamper {
         final Shader shader_;
         final Scaling scaling_;
+        final Subrange dataclip_;
         final Combiner combiner_;
 
         /**
@@ -1729,12 +1757,14 @@ public abstract class ShapeMode implements ModePlotter.Mode {
          *
          * @param   shader  colour shader
          * @param  scaling  count scaling strategy
+         * @param  dataclip  count scaling range adjustment
          * @param  combiner   combiner
          */
-        public WeightStamper( Shader shader, Scaling scaling,
+        public WeightStamper( Shader shader, Scaling scaling, Subrange dataclip,
                               Combiner combiner ) {
             shader_ = shader;
             scaling_ = scaling;
+            dataclip_ = dataclip;
             combiner_ = combiner;
         }
 
@@ -1749,6 +1779,7 @@ public abstract class ShapeMode implements ModePlotter.Mode {
                 WeightStamper other = (WeightStamper) o;
                 return this.shader_.equals( other.shader_ )
                     && this.scaling_.equals( other.scaling_ )
+                    && this.dataclip_.equals( other.dataclip_ )
                     && this.combiner_.equals( other.combiner_ );
             }
             else {
@@ -1761,6 +1792,7 @@ public abstract class ShapeMode implements ModePlotter.Mode {
             int code = 3311;
             code = 23 * code + shader_.hashCode();
             code = 23 * code + scaling_.hashCode();
+            code = 23 * code + dataclip_.hashCode();
             code = 23 * code + combiner_.hashCode();
             return code;
         }
@@ -1772,6 +1804,7 @@ public abstract class ShapeMode implements ModePlotter.Mode {
     public static class ShadeStamper implements Stamper {
         final Shader shader_;
         final Scaling scaling_;
+        final Subrange dataclip_;
         final Color baseColor_;
         final Color nullColor_;
         final float scaleAlpha_;
@@ -1781,6 +1814,7 @@ public abstract class ShapeMode implements ModePlotter.Mode {
          *
          * @param  shader  colour shader 
          * @param  scaling   scaling function from data to shade value
+         * @param  dataclip  scaling function range adjustment
          * @param  baseColor  colour to use for adjustments in case of
          *                    non-absolute shader
          * @param  nullColor  colour to use for null aux coordinate,
@@ -1788,10 +1822,12 @@ public abstract class ShapeMode implements ModePlotter.Mode {
          * @param  scaleAlpha  factor to scale output colour alpha by;
          *                     1 means opaque
          */
-        public ShadeStamper( Shader shader, Scaling scaling, Color baseColor,
-                             Color nullColor, float scaleAlpha ) {
+        public ShadeStamper( Shader shader, Scaling scaling, Subrange dataclip,
+                             Color baseColor, Color nullColor,
+                             float scaleAlpha ) {
             shader_ = shader;
             scaling_ = scaling;
+            dataclip_ = dataclip;
             baseColor_ = baseColor;
             nullColor_ = nullColor;
             scaleAlpha_ = scaleAlpha;
@@ -1808,6 +1844,7 @@ public abstract class ShapeMode implements ModePlotter.Mode {
                 ShadeStamper other = (ShadeStamper) o;
                 return this.shader_.equals( other.shader_ )
                     && this.scaling_.equals( other.scaling_ )
+                    && this.dataclip_.equals( other.dataclip_ )
                     && PlotUtil.equals( this.baseColor_, other.baseColor_ )
                     && PlotUtil.equals( this.nullColor_, other.nullColor_ )
                     && this.scaleAlpha_ == other.scaleAlpha_;
@@ -1822,6 +1859,7 @@ public abstract class ShapeMode implements ModePlotter.Mode {
             int code = 7301;
             code = 23 * code + shader_.hashCode();
             code = 23 * code + scaling_.hashCode();
+            code = 23 * code + dataclip_.hashCode();
             code = 23 * code + PlotUtil.hashCode( baseColor_ );
             code = 23 * code + PlotUtil.hashCode( nullColor_ );
             code = 23 * code + Float.floatToIntBits( scaleAlpha_ );
@@ -1865,7 +1903,7 @@ public abstract class ShapeMode implements ModePlotter.Mode {
         final DataGeom geom_;
         final DataSpec dataSpec_;
         final Outliner outliner_;
-        final Map<AuxScale,Range> auxRanges_;
+        final Map<AuxScale,Span> auxSpans_;
         final PaperType paperType_;
         final ShapePainter painter_;
 
@@ -1876,25 +1914,25 @@ public abstract class ShapeMode implements ModePlotter.Mode {
          * @param  geom   data position coordinate definition
          * @param  dataSpec  data specification
          * @param  outliner  outline shape
-         * @param  auxRanges   data ranges calculated by request
+         * @param  auxSpans   data ranges calculated by request
          * @param  paperType  graphics destination type
          */
         DrawSpec( Surface surface, DataGeom geom, DataSpec dataSpec,
-                  Outliner outliner, Map<AuxScale,Range> auxRanges,
+                  Outliner outliner, Map<AuxScale,Span> auxSpans,
                   PaperType paperType ) {
             surface_ = surface;
             geom_ = geom;
             dataSpec_ = dataSpec;
             outliner_ = outliner;
-            auxRanges_ = auxRanges;
+            auxSpans_ = auxSpans;
             paperType_ = paperType;
             if ( paperType instanceof PaperType2D ) {
-                painter_ = outliner.create2DPainter( surface, geom, auxRanges,
+                painter_ = outliner.create2DPainter( surface, geom, auxSpans,
                                                      (PaperType2D) paperType );
             }
             else if ( paperType instanceof PaperType3D ) {
                 painter_ = outliner.create3DPainter( (CubeSurface) surface,
-                                                     geom, auxRanges,
+                                                     geom, auxSpans,
                                                      (PaperType3D) paperType );
             }
             else {
@@ -1912,7 +1950,7 @@ public abstract class ShapeMode implements ModePlotter.Mode {
         public Object calculateBinPlan( Object[] knownPlans,
                                         DataStore dataStore ) {
             return outliner_
-                  .calculateBinPlan( surface_, geom_, auxRanges_, dataStore,
+                  .calculateBinPlan( surface_, geom_, auxSpans_, dataStore,
                                      dataSpec_, knownPlans );
         }
 
@@ -1979,11 +2017,11 @@ public abstract class ShapeMode implements ModePlotter.Mode {
         }
 
         public Drawing createDrawing( Surface surface,
-                                      Map<AuxScale,Range> auxRanges,
+                                      Map<AuxScale,Span> auxSpans,
                                       PaperType paperType ) {
             DrawSpec drawSpec =
                 new DrawSpec( surface, getDataGeom(), getDataSpec(), outliner_,
-                              auxRanges, paperType );
+                              auxSpans, paperType );
             return createDrawing( drawSpec );
         }
     }
