@@ -4,22 +4,27 @@ import java.awt.Graphics;
 import java.awt.Point;
 import java.awt.Rectangle;
 import java.awt.geom.Point2D;
+import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import javax.swing.Icon;
+import uk.ac.starlink.table.AbstractStarTable;
 import uk.ac.starlink.table.ColumnData;
 import uk.ac.starlink.table.ColumnInfo;
 import uk.ac.starlink.table.ColumnStarTable;
 import uk.ac.starlink.table.DefaultValueInfo;
+import uk.ac.starlink.table.HealpixTableInfo;
+import uk.ac.starlink.table.IteratorRowSequence;
+import uk.ac.starlink.table.RowSequence;
 import uk.ac.starlink.table.StarTable;
 import uk.ac.starlink.table.ValueInfo;
 import uk.ac.starlink.ttools.gui.ResourceIcon;
 import uk.ac.starlink.ttools.func.Tilings;
 import uk.ac.starlink.ttools.plot.Matrices;
-import uk.ac.starlink.ttools.plot.Range;
 import uk.ac.starlink.ttools.plot.Shader;
 import uk.ac.starlink.ttools.plot.Shaders;
 import uk.ac.starlink.ttools.plot.Style;
@@ -31,11 +36,14 @@ import uk.ac.starlink.ttools.plot2.Drawing;
 import uk.ac.starlink.ttools.plot2.LayerOpt;
 import uk.ac.starlink.ttools.plot2.PlotLayer;
 import uk.ac.starlink.ttools.plot2.Plotter;
+import uk.ac.starlink.ttools.plot2.Ranger;
 import uk.ac.starlink.ttools.plot2.ReportKey;
 import uk.ac.starlink.ttools.plot2.ReportMap;
 import uk.ac.starlink.ttools.plot2.ReportMeta;
 import uk.ac.starlink.ttools.plot2.Scaler;
 import uk.ac.starlink.ttools.plot2.Scaling;
+import uk.ac.starlink.ttools.plot2.Span;
+import uk.ac.starlink.ttools.plot2.Subrange;
 import uk.ac.starlink.ttools.plot2.Surface;
 import uk.ac.starlink.ttools.plot2.config.ConfigKey;
 import uk.ac.starlink.ttools.plot2.config.ConfigMap;
@@ -87,8 +95,8 @@ public class SkyDensityPlotter
     private static final ReportKey<Integer> ABSLEVEL_REPKEY =
         ReportKey
        .createIntegerKey( new ReportMeta( "abs_level",
-                                          "Absolute HEALPix Level" ),
-                          false );
+                                          "HEALPix Level" ),
+                          true );
     private static final ReportKey<Integer> RELLEVEL_REPKEY =
         ReportKey
        .createIntegerKey( new ReportMeta( "rel_level",
@@ -250,8 +258,8 @@ public class SkyDensityPlotter
         return sbuf.toString();
     }
 
-    public ConfigKey[] getStyleKeys() {
-        List<ConfigKey> keyList = new ArrayList<ConfigKey>();
+    public ConfigKey<?>[] getStyleKeys() {
+        List<ConfigKey<?>> keyList = new ArrayList<ConfigKey<?>>();
         keyList.add( LEVEL_KEY );
         if ( weightCoord_ != null ) {
             keyList.add( COMBINER_KEY );
@@ -263,20 +271,22 @@ public class SkyDensityPlotter
         if ( transparent_ ) {
             keyList.add( TRANSPARENCY_KEY );
         }
-        return keyList.toArray( new ConfigKey[ 0 ] );
+        return keyList.toArray( new ConfigKey<?>[ 0 ] );
     }
 
     public SkyDenseStyle createStyle( ConfigMap config ) {
         RampKeySet.Ramp ramp = RAMP_KEYS.createValue( config );
         int level = config.get( LEVEL_KEY );
         Scaling scaling = ramp.getScaling();
+        Subrange dataclip = ramp.getDataClip();
         float scaleAlpha = 1f - config.get( TRANSPARENCY_KEY ).floatValue();
         Shader shader = Shaders.fade( ramp.getShader(), scaleAlpha );
         Combiner combiner = weightCoord_ == null
                           ? Combiner.COUNT
                           : config.get( COMBINER_KEY );
         SolidAngleUnit unit = config.get( ANGLE_KEY );
-        return new SkyDenseStyle( level, scaling, shader, combiner, unit );
+        return new SkyDenseStyle( level, scaling, dataclip, shader, combiner,
+                                  unit );
     }
 
     public PlotLayer createLayer( DataGeom geom, DataSpec dataSpec,
@@ -291,7 +301,7 @@ public class SkyDensityPlotter
      * An attempt is made to return the result for the "largest" screen pixel
      * (the one covering more of the sky than any other).
      *
-     * @param  surface
+     * @param  surface  plot surface
      * @return  approximately corresponding HEALPix level
      */
     public static int getPixelLevel( SkySurface surface ) {
@@ -333,6 +343,7 @@ public class SkyDensityPlotter
 
         private final int level_;
         private final Scaling scaling_;
+        private final Subrange dataclip_;
         private final Shader shader_;
         private final Combiner combiner_;
         private final SolidAngleUnit unit_;
@@ -347,14 +358,17 @@ public class SkyDensityPlotter
          *                  than that defined by this level
          * @param   scaling   scaling function for mapping densities to
          *                    colour map entries
+         * @param   dataclip  scaling function adjustment subrange
          * @param   shader   colour map
          * @param   combiner  value combination mode for bin calculation
          * @param   unit     solid angle unit for density combinations
          */
-        public SkyDenseStyle( int level, Scaling scaling, Shader shader,
-                              Combiner combiner, SolidAngleUnit unit ) {
+        public SkyDenseStyle( int level, Scaling scaling, Subrange dataclip,
+                              Shader shader, Combiner combiner,
+                              SolidAngleUnit unit ) {
             level_ = level;
             scaling_ = scaling;
+            dataclip_ = dataclip;
             shader_ = shader;
             combiner_ = combiner;
             unit_ = unit;
@@ -381,6 +395,7 @@ public class SkyDensityPlotter
             int code = 23443;
             code = 23 * code + level_;
             code = 23 * code + scaling_.hashCode();
+            code = 23 * code + dataclip_.hashCode();
             code = 23 * code + shader_.hashCode();
             code = 23 * code + combiner_.hashCode();
             code = 23 * code + unit_.hashCode();
@@ -393,6 +408,7 @@ public class SkyDensityPlotter
                 SkyDenseStyle other = (SkyDenseStyle) o;
                 return this.level_ == other.level_
                     && this.scaling_.equals( other.scaling_ )
+                    && this.dataclip_.equals( other.dataclip_ )
                     && this.shader_.equals( other.shader_ )
                     && this.combiner_.equals( other.combiner_ )
                     && this.unit_.equals( other.unit_ );
@@ -433,11 +449,11 @@ public class SkyDensityPlotter
         }
 
         public Drawing createDrawing( Surface surface,
-                                      Map<AuxScale,Range> auxRanges,
+                                      Map<AuxScale,Span> auxSpans,
                                       PaperType paperType ) {
             SkySurface ssurf = (SkySurface) surface;
             return new SkyDensityDrawing( ssurf, createTileRenderer( ssurf ),
-                                          auxRanges.get( SCALE ), paperType );
+                                          auxSpans.get( SCALE ), paperType );
         }
 
         public Map<AuxScale,AuxReader> getAuxRangers() {
@@ -449,10 +465,13 @@ public class SkyDensityPlotter
                 public ValueInfo getAxisInfo( DataSpec dataSpec ) {
                     return getCombinedInfo( dataSpec );
                 }
+                public Scaling getScaling() {
+                    return dstyle_.scaling_;
+                }
                 public void adjustAuxRange( Surface surface, DataSpec dataSpec,
                                             DataStore dataStore,
                                             Object[] knownPlans,
-                                            Range range ) {
+                                            Ranger ranger ) {
                     SkySurface ssurf = (SkySurface) surface;
                     int level = getLevel( ssurf );
 
@@ -465,7 +484,7 @@ public class SkyDensityPlotter
                     SkyDensityPlan splan =
                         getSkyPlan( knownPlans, level, dataSpec );
                     if ( splan != null ) {
-                        splan.extendVisibleRange( range, ssurf,
+                        splan.extendVisibleRange( ranger, ssurf,
                                                   getBinFactor( level ) );
                     }
                     else {
@@ -473,7 +492,7 @@ public class SkyDensityPlotter
                             readBins( ssurf, dataSpec, dataStore )
                            .getResult();
                         createTileRenderer( ssurf )
-                       .extendAuxRange( range, binResult );
+                       .extendAuxRange( ranger, binResult );
                     }
                 }
             } );
@@ -536,16 +555,9 @@ public class SkyDensityPlotter
                                   DataStore dataStore ) {
             int level = getLevel( surface );
             SkyPixer skyPixer = new SkyPixer( level );
-            BinList binList = null;
             long npix = skyPixer.getPixelCount();
             Combiner combiner = dstyle_.combiner_;
-            if ( npix < 200000 ) {
-                binList = combiner.createArrayBinList( (int) npix );
-            }
-            if ( binList == null ) {
-                binList = combiner.createHashBinList( npix );
-            }
-            assert binList != null;
+            BinList binList = Combiner.createDefaultBinList( combiner, npix );
             TupleSequence tseq = dataStore.getTupleSequence( dataSpec );
             int icPos = coordGrp_.getPosCoordIndex( 0, geom_ );
             double[] v3 = new double[ 3 ];
@@ -628,7 +640,7 @@ public class SkyDensityPlotter
 
             private final SkySurface surface_;
             private final SkyTileRenderer renderer_;
-            private final Range auxRange_;
+            private final Span auxSpan_;
             private final PaperType paperType_;
             private final int level_;
             private final int pixelLevel_;
@@ -638,14 +650,14 @@ public class SkyDensityPlotter
              *
              * @param   surface  plot surface
              * @param   renderer  can plot healpix tiles
-             * @param   auxRange  range defining colour scaling
+             * @param   auxSpan   colour scaling range
              * @param   paperType  paper type
              */
             SkyDensityDrawing( SkySurface surface, SkyTileRenderer renderer,
-                               Range auxRange, PaperType paperType ) {
+                               Span auxSpan, PaperType paperType ) {
                 surface_ = surface;
                 renderer_ = renderer;
-                auxRange_ = auxRange;
+                auxSpan_ = auxSpan;
                 paperType_ = paperType;
                 level_ = getLevel( surface );
                 pixelLevel_ = getPixelLevel( surface );
@@ -682,7 +694,8 @@ public class SkyDensityPlotter
                 final BinList.Result binResult =
                     ((SkyDensityPlan) plan).binResult_;
                 final Scaler scaler =
-                    Scaling.createRangeScaler( dstyle_.scaling_, auxRange_ );
+                    auxSpan_.createScaler( dstyle_.scaling_,
+                                           dstyle_.dataclip_ );
                 final Shader shader = dstyle_.shader_;
                 paperType_.placeDecal( paper, new Decal() {
                     public void paintDecal( Graphics g ) {
@@ -713,50 +726,141 @@ public class SkyDensityPlotter
 
         /**
          * Returns a StarTable containing the HEALPix bin information
-         * based on a given plan.  Calling this method is not expensive.
+         * based on a given plan.  Calling this method is not expensive,
+         * though serializing the resulting table may be.
+         *
+         * <p>Note that the exported table is based on the HEALPix level
+         * currently plotted, which may not be what the user asked for
+         * (since it doesn't bother to prepare pixels that are too small
+         * to see).
          *
          * @param   splan  plan representing plotted density map
          * @return   table suitable for export
          */
         private StarTable createExportTable( SkyDensityPlan splan ) {
             int level = splan.level_;
-            if ( level > 13 ) {
-                return null;
-            }
-            SkyPixer skyPixer = new SkyPixer( level );
+            final BinList.Result binResult = splan.binResult_;
+            SkyPixer pixer = new SkyPixer( level );
+            boolean isNested = pixer.isNested();
+            HealpixTableInfo.HpxCoordSys csys =
+                HealpixSys.fromGeom( geom_.getViewSystem() );
             DataSpec dataSpec = splan.dataSpec_;
-            Combiner combiner = splan.combiner_;
-            BinList.Result binResult = splan.binResult_;
+            long nsky = pixer.getPixelCount();
+            final long ndata = binResult.getBinCount();
+            final ColumnData dataCol =
+                BinResultColumnData
+               .createInstance( getCombinedInfo( dataSpec ), binResult,
+                                getBinFactor( level ) );
 
-            /* Construct a column containing HEALPix bin indices.
-             * In fact this is a bit superfluous, since the bin indices
-             * are just the same as the row numbers. */
-            String indexDescrip = "HEALPix index, level " + level + ", "
-                                + ( skyPixer.isNested() ? "Nested" : "Ring" )
-                                + " scheme";
-            ColumnInfo indexInfo =
-                new ColumnInfo( "hpx" + level, Integer.class, indexDescrip );
-            ColumnData indexCol = new ColumnData( indexInfo ) {
-                public Object readValue( long irow ) {
-                    return new Integer( (int) irow );
-                }
-            };
+            /* Full sky table. */
+            if ( ndata * 1.0 / nsky > 0.5 && nsky <= Integer.MAX_VALUE ) {
+                int nrow = (int) nsky;
+                assert nrow == nsky;
+                ColumnStarTable table =
+                    ColumnStarTable.makeTableWithRows( nrow );
+                table.addColumn( dataCol );
+                HealpixTableInfo hpxInfo =
+                    new HealpixTableInfo( level, isNested, null, csys );
+                table.getParameters()
+                     .addAll( Arrays.asList( hpxInfo.toParams() ) );
+                return table;
+            }
 
-            /* Construct a column containing bin contents, given that the
-             * bin index is row number. */
-            ValueInfo dataInfo = getCombinedInfo( dataSpec );
-            ColumnData dataCol =
-                BinResultColumnData.createInstance( dataInfo, binResult,
-                                                    getBinFactor( level ) );
+            /* Partial sky table. */
+            else {
+                String indexDescrip = "HEALPix index, level " + level + ", "
+                                    + ( isNested ? "Nested" : "Ring" )
+                                    + " scheme";
+                final boolean isLong = nsky > Integer.MAX_VALUE;
+                ColumnInfo indexInfo =
+                    new ColumnInfo( "hpx" + level,
+                                    isLong ? Long.class : Integer.class,
+                                    indexDescrip );
+                final ColumnInfo[] infos = new ColumnInfo[] {
+                    indexInfo,
+                    dataCol.getColumnInfo(),
+                };
+                StarTable table = new AbstractStarTable() {
+                    public ColumnInfo getColumnInfo( int icol ) {
+                        return infos[ icol ];
+                    }
+                    public int getColumnCount() {
+                        return infos.length;
+                    }
+                    public long getRowCount() {
+                        return ndata;
+                    }
+                    public RowSequence getRowSequence() {
+                        Iterator<Object[]> rowIt =
+                            new BinRowIterator( binResult.indexIterator(),
+                                                dataCol, isLong );
+                        return new IteratorRowSequence( rowIt );
+                    }
+                };
+                HealpixTableInfo hpxInfo =
+                    new HealpixTableInfo( level, isNested, indexInfo.getName(),
+                                          csys );
+                table.getParameters()
+                     .addAll( Arrays.asList( hpxInfo.toParams() ) );
+                return table;
+            }
+        }
+    }
 
-            /* Combine these into a table and return. */
-            final long nrow = skyPixer.getPixelCount();
-            assert (int) nrow == nrow;
-            ColumnStarTable table =
-                ColumnStarTable.makeTableWithRows( (int) nrow );
-            table.addColumn( indexCol );
-            table.addColumn( dataCol );
-            return table;
+    /**
+     * Iterator over Object[2] values giving an (index, value) pair
+     * for each non-blank entry in a BinList.Result.
+     * Suitable for use in an IteratorRowSequence;
+     */
+    private static class BinRowIterator implements Iterator<Object[]> {
+        private final Iterator<Long> indexIt_;
+        private final ColumnData dataCol_;
+        private final boolean isLong_;
+
+        /**
+         * Constructor.
+         *
+         * @param  indexIt  iterator over pixel indices to include
+         * @param  dataCol  contains data values at the row of each pixel index
+         * @param  isLong   if true, output index number is a Long;
+         *                  if false, it's an Integer
+         */
+        BinRowIterator( Iterator<Long> indexIt, ColumnData dataCol,
+                        boolean isLong ) {
+            indexIt_ = indexIt;
+            dataCol_ = dataCol;
+            isLong_ = isLong;
+        }
+
+        public Object[] next() {
+            Long index = indexIt_.next();
+            long ix = index.longValue();
+            final Object ixObj;
+
+            /* Careful: silent unboxing can do horrible things here.
+             * Evaluating "ixObj = isLong_ ? index : new Integer( (int) ix )"
+             * gives you a Long even when isLong_ false!! */
+            if ( isLong_ ) {
+                ixObj = index;
+            }
+            else {
+                ixObj = new Integer( (int) ix );
+            }
+
+            final Object dataObj;
+            try {
+                dataObj = dataCol_.readValue( ix );
+            }
+            catch ( IOException e ) {  // shouldn't happen
+                throw new IteratorRowSequence.PackagedIOException( e );
+            }
+            return new Object[] { ixObj, dataObj };
+        }
+        public boolean hasNext() {
+            return indexIt_.hasNext();
+        }
+        public void remove() {
+            throw new UnsupportedOperationException();
         }
     }
 
@@ -812,7 +916,7 @@ public class SkyDensityPlotter
         }
 
         /**
-         * Adjusts a supplied Range object to reflect the pixel value range
+         * Adjusts a supplied Ranger object to reflect the pixel value range
          * represented by this plan over
          * only that part of the sky visible in a given sky surface.
          * This may not be very fast, but it scales with the number of
@@ -821,11 +925,11 @@ public class SkyDensityPlotter
          * However in some cases (bin-count&lt;&lt;pixel-count)
          * it could definitely be done more efficiently.
          *
-         * @param  range     range to submit values to
+         * @param  ranger     ranger to submit values to
          * @param  surface   viewing surface
          * @param  binFactor   factor for multiplying bin result values
          */
-        public void extendVisibleRange( Range range, SkySurface surface,
+        public void extendVisibleRange( Ranger ranger, SkySurface surface,
                                         double binFactor ) {
             Rectangle bounds = surface.getPlotBounds();
             int nx = bounds.width;
@@ -844,7 +948,7 @@ public class SkyDensityPlotter
                     double dval =
                         binFactor *
                         binResult_.getBinValue( skyPixer.getIndex( dpos ) );
-                    range.submit( dval );
+                    ranger.submitDatum( dval );
                 }
             }
         }
