@@ -7,7 +7,6 @@ import java.awt.Rectangle;
 import java.awt.geom.Point2D;
 import javax.swing.Icon;
 import java.util.Map;
-import uk.ac.starlink.ttools.plot.Range;
 import uk.ac.starlink.ttools.plot.Style;
 import uk.ac.starlink.ttools.plot2.AuxScale;
 import uk.ac.starlink.ttools.plot2.DataGeom;
@@ -17,6 +16,7 @@ import uk.ac.starlink.ttools.plot2.LayerOpt;
 import uk.ac.starlink.ttools.plot2.Pixer;
 import uk.ac.starlink.ttools.plot2.PlotLayer;
 import uk.ac.starlink.ttools.plot2.PlotUtil;
+import uk.ac.starlink.ttools.plot2.Span;
 import uk.ac.starlink.ttools.plot2.Surface;
 import uk.ac.starlink.ttools.plot2.config.ConfigKey;
 import uk.ac.starlink.ttools.plot2.config.ConfigMap;
@@ -31,6 +31,7 @@ import uk.ac.starlink.ttools.plot2.paper.Paper;
 import uk.ac.starlink.ttools.plot2.paper.PaperType;
 import uk.ac.starlink.ttools.plot2.paper.PaperType2D;
 import uk.ac.starlink.ttools.plot2.paper.PaperType3D;
+import uk.ac.starlink.ttools.plot2.task.SimpleLayerType;
 
 /**
  * Fairly minimal example plotter implementation.
@@ -66,8 +67,8 @@ public class SpotPlotter extends AbstractPlotter<SpotPlotter.SpotStyle> {
         } );
     }
 
-    public ConfigKey[] getStyleKeys() {
-        return new ConfigKey[] {
+    public ConfigKey<?>[] getStyleKeys() {
+        return new ConfigKey<?>[] {
             StyleKeys.COLOR,
         };
     }
@@ -82,7 +83,7 @@ public class SpotPlotter extends AbstractPlotter<SpotPlotter.SpotStyle> {
         LayerOpt opt = new LayerOpt( style.getColor(), true );
         return new AbstractPlotLayer( this, geom, dataSpec, style, opt ) {
             public Drawing createDrawing( final Surface surface,
-                                          Map<AuxScale,Range> auxRanges,
+                                          Map<AuxScale,Span> auxSpans,
                                           PaperType paperType ) {
                 if ( paperType instanceof PaperType2D ) {
                     final PaperType2D ptype = (PaperType2D) paperType;
@@ -227,7 +228,8 @@ public class SpotPlotter extends AbstractPlotter<SpotPlotter.SpotStyle> {
                 g.fillRect( -1, -1, 3, 3 );
             }
             public Pixer createPixer( Rectangle clip ) {
-                return Pixers.createArrayPixer( xs, ys, np );
+                return Pixers.clip( Pixers.createArrayPixer( xs, ys, np ),
+                                    clip );
             }
         };
     }
@@ -261,4 +263,13 @@ public class SpotPlotter extends AbstractPlotter<SpotPlotter.SpotStyle> {
             return color_.hashCode();
         }
     } 
+
+    /**
+     * LayerType corresponding to SpotPlotter.
+     */
+    public static class SpotLayerType extends SimpleLayerType {
+        public SpotLayerType() {
+            super( new SpotPlotter() );
+        }
+    }
 }
