@@ -14,7 +14,6 @@ import java.util.List;
 import java.util.Map;
 import javax.swing.Icon;
 import uk.ac.starlink.ttools.gui.ResourceIcon;
-import uk.ac.starlink.ttools.plot.Range;
 import uk.ac.starlink.ttools.plot2.AuxScale;
 import uk.ac.starlink.ttools.plot2.Axis;
 import uk.ac.starlink.ttools.plot2.DataGeom;
@@ -27,6 +26,7 @@ import uk.ac.starlink.ttools.plot2.Plotter;
 import uk.ac.starlink.ttools.plot2.ReportKey;
 import uk.ac.starlink.ttools.plot2.ReportMap;
 import uk.ac.starlink.ttools.plot2.ReportMeta;
+import uk.ac.starlink.ttools.plot2.Span;
 import uk.ac.starlink.ttools.plot2.Surface;
 import uk.ac.starlink.ttools.plot2.config.BooleanConfigKey;
 import uk.ac.starlink.ttools.plot2.config.ConfigKey;
@@ -177,8 +177,8 @@ public class Stats1Plotter implements Plotter<Stats1Plotter.StatsStyle> {
         return fitCoordGrp_;
     }
 
-    public ConfigKey[] getStyleKeys() {
-        List<ConfigKey> list = new ArrayList<ConfigKey>();
+    public ConfigKey<?>[] getStyleKeys() {
+        List<ConfigKey<?>> list = new ArrayList<ConfigKey<?>>();
         list.add( StyleKeys.COLOR );
         list.add( SHOWMEAN_KEY );
         list.addAll( Arrays.asList( StyleKeys.getStrokeKeys() ) );
@@ -188,7 +188,7 @@ public class Stats1Plotter implements Plotter<Stats1Plotter.StatsStyle> {
         }
         list.add( NORMALISE_KEY );
         list.add( BINSIZER_KEY );
-        return list.toArray( new ConfigKey[ 0 ] );
+        return list.toArray( new ConfigKey<?>[ 0 ] );
     }
 
     public StatsStyle createStyle( ConfigMap config ) {
@@ -210,7 +210,7 @@ public class Stats1Plotter implements Plotter<Stats1Plotter.StatsStyle> {
         return new AbstractPlotLayer( this, fitDataGeom_, dataSpec,
                                       style, layerOpt ) {
             public Drawing createDrawing( Surface surface,
-                                          Map<AuxScale,Range> auxRanges,
+                                          Map<AuxScale,Span> auxSpans,
                                           PaperType paperType ) {
                 return new StatsDrawing( (PlanarSurface) surface, geom,
                                          dataSpec, style, paperType );
@@ -425,6 +425,7 @@ public class Stats1Plotter implements Plotter<Stats1Plotter.StatsStyle> {
             int gxhi = box.x + box.width + 2;
             int np = gxhi - gxlo;
             LineTracer tracer = style.createLineTracer( g2, box, np, isBitmap );
+            Color color = style.getColor();
             Point2D.Double gpos = new Point2D.Double();
             double[] dpos = new double[ surface.getDataDimCount() ];
             for ( int ip = 0; ip < np; ip++ ) {
@@ -436,7 +437,7 @@ public class Stats1Plotter implements Plotter<Stats1Plotter.StatsStyle> {
                     dpos[ 1 ] = factor * gaussian( dx );
                     if ( surface.dataToGraphics( dpos, false, gpos ) &&
                          PlotUtil.isPointReal( gpos ) ) {
-                        tracer.addVertex( gpos.x, gpos.y );
+                        tracer.addVertex( gpos.x, gpos.y, color );
                     }
                 }
             }
@@ -449,8 +450,8 @@ public class Stats1Plotter implements Plotter<Stats1Plotter.StatsStyle> {
                 double gyhi = axes[ 1 ].dataToGraphics( factor );
                 LineTracer meanTracer =
                     style.createLineTracer( g2, box, 3, isBitmap );
-                meanTracer.addVertex( gx, gylo );
-                meanTracer.addVertex( gx, gyhi );
+                meanTracer.addVertex( gx, gylo, color );
+                meanTracer.addVertex( gx, gyhi, color );
                 meanTracer.flush();
             }
         }
