@@ -10,6 +10,8 @@ import java.util.List;
 import uk.ac.starlink.table.ColumnData;
 import uk.ac.starlink.table.ColumnStarTable;
 import uk.ac.starlink.table.DefaultValueInfo;
+import uk.ac.starlink.table.DescribedValue;
+import uk.ac.starlink.table.HealpixTableInfo;
 import uk.ac.starlink.table.RowSequence;
 import uk.ac.starlink.table.StarTable;
 import uk.ac.starlink.table.Tables;
@@ -22,6 +24,7 @@ import uk.ac.starlink.task.Parameter;
 import uk.ac.starlink.task.ParameterValueException;
 import uk.ac.starlink.task.StringParameter;
 import uk.ac.starlink.task.TaskException;
+import uk.ac.starlink.ttools.cone.HealpixTiling;
 import uk.ac.starlink.ttools.cone.SkyTiling;
 import uk.ac.starlink.ttools.cone.TilingParameter;
 import uk.ac.starlink.ttools.jel.JELQuantity;
@@ -55,7 +58,6 @@ public class SkyDensityMap extends SingleMapperTask {
     private final ChoiceParameter<Combiner> combinerParam_;
     private final ChoiceParameter<SolidAngleUnit> unitParam_;
     private final BooleanParameter completeParam_;
-    private static final int MAX_ARRAY = 1000000;
 
     /**
      * Constructor.
@@ -200,7 +202,7 @@ public class SkyDensityMap extends SingleMapperTask {
         } );
         quantParam_.setNullPermitted( true );
 
-        getParameterList().addAll( Arrays.asList( new Parameter[] {
+        getParameterList().addAll( Arrays.asList( new Parameter<?>[] {
             lonParam_,
             latParam_,
             tilingParam_,
@@ -261,6 +263,18 @@ public class SkyDensityMap extends SingleMapperTask {
         final SingleTableMapping mapping =
             new SkyMapMapping( lonString, latString, tiling, complete, aqs,
                                countIndex );
+        final DescribedValue[] params;
+        if ( tiling instanceof HealpixTiling ) {
+            HealpixTiling hpx = (HealpixTiling) tiling;
+            HealpixTableInfo.HpxCoordSys csys = null;
+            HealpixTableInfo hpxInfo =
+                new HealpixTableInfo( hpx.getHealpixK(), hpx.isNest(),
+                                      hpx.getIndexInfo().getName(), csys );
+            params = hpxInfo.toParams();
+        }
+        else {
+            params = new DescribedValue[ 0 ];
+        }
         final TableProducer inProd = createInputProducer( env );
         return new TableProducer() {
             public StarTable getTable() throws IOException, TaskException {
@@ -335,7 +349,8 @@ public class SkyDensityMap extends SingleMapperTask {
                 Combiner combiner = aq.combiner_;
                 String expr = aq.expr_;
                 SolidAngleUnit unit = aq.unit_;
-                BinList binList = createBinList( combiner, npix, complete_ );
+                BinList binList =
+                    Combiner.createDefaultBinList( combiner, npix );
                 JELQuantity jq;
                 try {
                     jq = JELUtils.compileQuantity( lib, jelReader, expr,
@@ -385,13 +400,14 @@ public class SkyDensityMap extends SingleMapperTask {
             for ( Binner binner : binners ) {
                 binsTable.addColumn( binner.createColumnData( tiling_ ) );
             }
+            final StarTable outTable;
 
-            /* Either output the table as is, with one row for each pixel. */
+            /* Either use the table as is, with one row for each pixel. */
             if ( complete_ ) {
-                return binsTable;
+                outTable = binsTable;
             }
 
-            /* Or output a compressed version, where only those rows with
+            /* Or prepare a compressed version, where only those rows with
              * some data in the non-pixel-index columns are included. */
             else {
                 int icolOffset = 1; // pixel index column
@@ -406,9 +422,22 @@ public class SkyDensityMap extends SingleMapperTask {
                         testIcols[ i ] = icolOffset + i;
                     }
                 }
-                return new UnsparseTable( binsTable, minIndex, maxIndex,
-                                          testIcols );
+                outTable = new UnsparseTable( binsTable, minIndex, maxIndex,
+                                              testIcols );
             }
+
+            /* Add HEALPix-specific metadata if applicable. */
+            if ( tiling_ instanceof HealpixTiling ) {
+                HealpixTiling hpxTiling = (HealpixTiling) tiling_;
+                HealpixTableInfo hpxInfo =
+                    new HealpixTableInfo( hpxTiling.getHealpixK(),
+                                          hpxTiling.isNest(),
+                                          hpxTiling.getIndexInfo().getName(),
+                                          (HealpixTableInfo.HpxCoordSys) null );
+                outTable.getParameters()
+                        .addAll( Arrays.asList( hpxInfo.toParams() ) );
+            }
+            return outTable;
         }
     }
 
@@ -431,25 +460,6 @@ public class SkyDensityMap extends SingleMapperTask {
         }
     }
 
-    /**
-     * Creates an accumulator for a given combination mode.
-     *
-     * @param  combiner  combiner
-     * @param  npix    (maximum) number of bins required
-     * @param  isComplete  hint about whether values are likely to be
-     *                     caculated for all pixels
-     */
-    private static BinList createBinList( Combiner combiner, long npix,
-                                          boolean isComplete ) {
-        if ( isComplete && npix < MAX_ARRAY ) {
-            BinList binList = combiner.createArrayBinList( (int) npix );
-            if ( binList != null ) {
-                return binList;
-            }
-        }
-        return combiner.createHashBinList( npix );
-    }
-
     /**
      * Creates a table column that reports pixel index for a given tiling.
      * The data content is just equivalent to the (zero-based) row index,
