@@ -291,6 +291,19 @@ public class FuncTest extends TestCase {
         }
         assertArrayEquals( new String[] { "Armadillo", "Scout", },
                            Arrays.pick( sa, 1, -1 ) );
+
+        assertEquals( 1, Arrays.indexOf( sa, "Armadillo" ) );
+        assertEquals( -1, Arrays.indexOf( sa, "Housepop" ) );
+        assertEquals( 5, Arrays.indexOf( da, 15 ) );
+        assertEquals( -1, Arrays.indexOf( da, Double.NaN ) );
+        assertEquals( -1, Arrays.indexOf( da, 99.e9 ) );
+        assertEquals( 5, Arrays.indexOf( ia, 15 ) );
+        assertEquals( -1, Arrays.indexOf( ia, (short) -1 ) );
+
+        assertEquals( 1, Arrays.indexOf(Arrays.stringArray("QSO", "BCG", "SNR"),
+                                        "BCG"));
+        assertEquals(-1, Arrays.indexOf(Arrays.stringArray("QSO", "BCG", "SNR"),
+                                        "TLA"));
     }
 
     public void testConversions() {
@@ -342,6 +355,18 @@ public class FuncTest extends TestCase {
         catch ( NumberFormatException e ) {
         }
 
+        assertArrayEquals( new int[] { 9, 8, -23, },
+                           Conversions.parseInts( "9 8 -23" ) );
+        assertEquals( 0, Conversions.parseInts( "tiddly-pom" ).length );
+
+        assertArrayEquals( new double[] { 1.3, 990.0, Double.NaN, -23.0 },
+                           Conversions.parseDoubles( "1.3, 99e1, NaN, -23" ) );
+        assertArrayEquals( new double[] { 0.8, 2.1, 9.0, 2.1, 6.2, 8.6 },
+             Conversions.parseDoubles("POLYGON(0.8, 2.1, 9.0, 2.1, 6.2, 8.6)"));
+        assertArrayEquals( new double[] { 0.8, 2.1, 9.0, 2.1, 6.2, 8.6 },
+             Conversions.parseDoubles("Polygon ICRS 0.8 2.1 9.0 2.1 6.2 8.6"));
+        assertEquals( 0, Conversions.parseDoubles( "La la la" ).length );
+
         assertEquals( (byte) 3, Conversions.toByte( 3.99 ) );
         try {
             Conversions.toByte( -190 );
@@ -613,6 +638,33 @@ public class FuncTest extends TestCase {
         assertEquals( null, Strings.desigToIcrs( "NGC 4993" ) );
         assertTrue( Double.isNaN( Strings.desigToRa( "HIP Z190012-230210" ) ) );
         assertTrue( Double.isNaN( Strings.desigToDec( "TYC J190012230210" ) ) );
+
+        assertEquals( "https://ui.adsabs.harvard.edu/abs/2018A%26A...616A...2L",
+                      Strings.bibcodeUrl( "2018A&A...616A...2L" ) );
+        assertNull( Strings.bibcodeUrl( null ) );
+        assertNull( Strings.bibcodeUrl( "Fredor" ) );
+        assertEquals( "https://doi.org/10.3390/informatics4030018",
+                      Strings.doiUrl( "10.3390/informatics4030018" ) );
+        assertEquals( "https://doi.org/10.3390/informatics4030018",
+                      Strings.doiUrl( "doi:10.3390/informatics4030018" ) );
+        assertNull( Strings.doiUrl( "Fredor" ) );
+        assertNull( Strings.doiUrl( null ) );
+        assertEquals( "https://arxiv.org/abs/1804.09379",
+                      Strings.arxivUrl( "1804.09379" ) );
+        assertEquals( "https://arxiv.org/abs/1804.09379",
+                      Strings.arxivUrl( "arXiv:1804.09379" ) );
+        assertNull( Strings.arxivUrl( null ) );
+        assertNull( Strings.arxivUrl( "Fredor" ) );
+
+        assertEquals( "http://simbad.u-strasbg.fr/simbad/sim-id?Ident="
+                    + "Beta%20Pictoris",
+                      Strings.simbadUrl( "Beta Pictoris" ) );
+        assertNull( Strings.simbadUrl( null ) );
+        assertNull( Strings.simbadUrl( " " ) );
+        assertEquals( "http://ned.ipac.caltech.edu/byname?objname=NGC%203952",
+                      Strings.nedUrl( "NGC 3952" ) );
+        assertNull( Strings.nedUrl( null ) );
+        assertNull( Strings.nedUrl( "    " ) );
     }
 
     private void checkDesig( String raSex, String decSex, String desig ) {
